//*****************************************************************************
//*
//*
//*		BinaryCodec.cpp
//*
//*
//*****************************************************************************
#include	<stdio.h>
#include	<string.h>
#include	"StringChain.h"
#include	"FileFunctions.h"

#include	"BinaryCodec.h"


#define 	OSL_ENSURE(a,b)


//*****************************************************************************
//*
//*		BinaryCodec_RCF
//*
//*****************************************************************************
BinaryCodec_RCF::BinaryCodec_RCF()
{
	mhCipher = rtl_cipher_create( rtl_Cipher_AlgorithmARCFOUR, rtl_Cipher_ModeStream );
	OSL_ENSURE( mhCipher != 0, "BinaryCodec_RCF::BinaryCodec_RCF - cannot create cipher" );

	mhDigest = rtl_digest_create( rtl_Digest_AlgorithmMD5 );
	OSL_ENSURE( mhDigest != 0, "BinaryCodec_RCF::BinaryCodec_RCF - cannot create digest" );

	memset( mpnDigestValue, 0, sizeof( mpnDigestValue ) );
}

//*****************************************************************************
//*
//*		BinaryCodec_RCF
//*
//*****************************************************************************
BinaryCodec_RCF::~BinaryCodec_RCF()
{
	(void)memset( mpnDigestValue, 0, sizeof( mpnDigestValue ) );
	rtl_digest_destroy( mhDigest );
	rtl_cipher_destroy( mhCipher );
}

//*****************************************************************************
//*
//*		initKey
//*
//*****************************************************************************
void BinaryCodec_RCF::initKey( const sal_uInt16 pnPassData[ 16 ],const sal_uInt8 pnUnique[ 16 ] )
{
	// create little-endian key data array from password data
	sal_uInt8		  pnKeyData[ 64 ];
	const sal_uInt16* pnCurrPass = pnPassData;
	const sal_uInt16* pnPassEnd	 = pnPassData + 16;
	sal_uInt8*		  pnCurrKey	 = pnKeyData;
	size_t			  nPassSize	 = 0;


	memset( pnKeyData, 0, sizeof( pnKeyData ) );



	for(;(pnCurrPass<pnPassEnd)&&(*pnCurrPass!=0);pnCurrPass++,nPassSize++)
		{
		*pnCurrKey++ = static_cast< sal_uInt8 >( *pnCurrPass );
		*pnCurrKey++ = static_cast< sal_uInt8 >( *pnCurrPass >> 8 );
		}

	pnKeyData[2 * nPassSize] = 0x80;
	pnKeyData[56           ] = static_cast< sal_uInt8 >( nPassSize << 4 );

	// fill raw digest of key data into key data
	(void)rtl_digest_updateMD5( mhDigest, pnKeyData, sizeof( pnKeyData ) );
	(void)rtl_digest_rawMD5	  ( mhDigest, pnKeyData, RTL_DIGEST_LENGTH_MD5 );

	// update digest with key data and passed unique data
	for(size_t nIndex=0;nIndex<16;nIndex++)
		{
		rtl_digest_updateMD5(mhDigest,pnKeyData, 5);
		rtl_digest_updateMD5(mhDigest,pnUnique ,16);
		}

	// update digest with padding
	pnKeyData[ 16 ] = 0x80;
	(void)memset( pnKeyData + 17, 0, sizeof( pnKeyData ) - 17 );
	pnKeyData[56] = 0x80;
	pnKeyData[57] = 0x0A;

	rtl_digest_updateMD5( mhDigest, pnKeyData + 16, sizeof( pnKeyData ) - 16 );

	// fill raw digest of above updates into digest value
	rtl_digest_rawMD5( mhDigest, mpnDigestValue, sizeof( mpnDigestValue ) );

	
}

//*****************************************************************************
//*
//*		initCipher
//*
//*****************************************************************************
bool BinaryCodec_RCF::initCipher (sal_uInt32 nCounter)
{
    rtlCipherError result;
    sal_uInt8      pKeyData[64]; // 512-bit message block

    // Initialize KeyData array.
    (void)memset (pKeyData, 0, sizeof(pKeyData));

    // Fill 40 bit of DigestValue into [0..4].
    (void)memcpy (pKeyData, mpnDigestValue, 5);

    // Fill counter into [5..8].
    pKeyData[ 5] = sal_uInt8((nCounter >>  0) & 0xff);
    pKeyData[ 6] = sal_uInt8((nCounter >>  8) & 0xff);
    pKeyData[ 7] = sal_uInt8((nCounter >> 16) & 0xff);
    pKeyData[ 8] = sal_uInt8((nCounter >> 24) & 0xff);

    pKeyData[ 9] = 0x80;
    pKeyData[56] = 0x48;

    // Fill raw digest of KeyData into KeyData.
    (void)rtl_digest_updateMD5 ( mhDigest, pKeyData, sizeof(pKeyData));
    (void)rtl_digest_rawMD5    ( mhDigest, pKeyData, RTL_DIGEST_LENGTH_MD5);

    // Initialize Cipher with KeyData (for decoding).
    result = rtl_cipher_init  ( mhCipher, rtl_Cipher_DirectionBoth, pKeyData, RTL_DIGEST_LENGTH_MD5, 0, 0);

    // Erase KeyData array and leave.
    (void)memset (pKeyData, 0, sizeof(pKeyData));

    return (result == rtl_Cipher_E_None);
}

//*****************************************************************************
//*
//*		verifyKey
//*
//*****************************************************************************
bool BinaryCodec_RCF::verifyKey( const sal_uInt8 pnSaltData[ 16 ],const sal_uInt8 pnSaltDigest[ 16 ] )
{
	if(!startBlock( 0 ))
		{
		return false;
		}

	sal_uInt8 pnDigest[ RTL_DIGEST_LENGTH_MD5 ];
	sal_uInt8 pnBuffer[ 64 ];

																// decode salt data into buffer
	rtl_cipher_decode( mhCipher, pnSaltData, 16, pnBuffer, sizeof( pnBuffer ) );

	pnBuffer[ 16 ] = 0x80;
	(void)memset( pnBuffer + 17, 0, sizeof( pnBuffer ) - 17 );
	pnBuffer[ 56 ] = 0x80;

	// fill raw digest of buffer into digest
	rtl_digest_updateMD5(mhDigest,pnBuffer,sizeof( pnBuffer) );
	rtl_digest_rawMD5   (mhDigest,pnDigest,sizeof( pnDigest) );

	// decode original salt digest into buffer
	rtl_cipher_decode( mhCipher, pnSaltDigest, 16, pnBuffer, sizeof( pnBuffer ) );

	// compare buffer with computed digest
	bool bResult = memcmp( pnBuffer, pnDigest, sizeof( pnDigest ) ) == 0;



return bResult;
}

//*****************************************************************************
//*
//*		startBlock
//*
//*****************************************************************************
bool BinaryCodec_RCF::startBlock( sal_Int32 nCounter )
{
	// initialize key data array
	sal_uInt8 pnKeyData[ 64 ];
	(void)memset( pnKeyData, 0, sizeof( pnKeyData ) );

	// fill 40 bit of digest value into [0..4]
	(void)memcpy( pnKeyData, mpnDigestValue, 5 );

	// fill little-endian counter into [5..8], static_cast masks out unneeded bits
	pnKeyData[ 5] = static_cast< sal_uInt8 >( nCounter );
	pnKeyData[ 6] = static_cast< sal_uInt8 >( nCounter >> 8 );
	pnKeyData[ 7] = static_cast< sal_uInt8 >( nCounter >> 16 );
	pnKeyData[ 8] = static_cast< sal_uInt8 >( nCounter >> 24 );

	pnKeyData[ 9] = 0x80;
	pnKeyData[56] = 0x48;

	// fill raw digest of key data into key data
	(void)rtl_digest_updateMD5( mhDigest, pnKeyData, sizeof( pnKeyData ) );
	(void)rtl_digest_rawMD5( mhDigest, pnKeyData, RTL_DIGEST_LENGTH_MD5 );

	// initialize cipher with key data (for decoding)
	rtlCipherError eResult =
	rtl_cipher_init( mhCipher, rtl_Cipher_DirectionDecode, pnKeyData, RTL_DIGEST_LENGTH_MD5, 0, 0 );



return eResult == rtl_Cipher_E_None;
}

//*****************************************************************************
//*
//*		decode
//*
//*****************************************************************************
bool BinaryCodec_RCF::decode( sal_uInt8 *pnDestData,const sal_uInt8 *pnSrcData,sal_Int32 nBytes )
{
rtlCipherError	eResult = rtl_cipher_decode( mhCipher,pnSrcData, static_cast< sal_Size >( nBytes ),pnDestData, static_cast< sal_Size >( nBytes ) );

return eResult == rtl_Cipher_E_None;
}

//*****************************************************************************
//*
//*		skip
//*
//*****************************************************************************
bool BinaryCodec_RCF::skip( sal_Int32 nBytes )
{
sal_uInt8	pnDummy[ 1024 ];
sal_Int32	nBytesLeft = nBytes;
bool		bResult    = true;

		// decode dummy data in memory to update internal state of RC4 cipher
	
	
	while(bResult && (nBytesLeft > 0))
		{
		sal_Int32	nBlockLen = min( nBytesLeft, static_cast< sal_Int32 >( sizeof( pnDummy ) ) );


		bResult 	= decode( pnDummy, pnDummy, nBlockLen );
		nBytesLeft -= nBlockLen;
		}

return bResult;
}

