//*****************************************************************************
//*
//*
//*		FileBuffer.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//

#ifndef __FILEBUFFER_H__
#define __FILEBUFFER_H__


#ifdef	_MSC_VER
#if 	_MSC_VER >= 1000
#pragma  once
#endif
#endif		


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if		_MSC_VER > 1299
	#ifndef		WINVER
	#define		WINVER 	0x0501
	#endif
	#endif
	#include	<Afx.h>
	#include	<AfxWin.h>
#elif !defined(LINUX) &&  !defined(linux)
	#include	<Windows.h>
#endif


#if defined(LINUX) || defined(linux)
	#define	 _cdecl
	#define	__cdecl
	#ifndef ifb64
	#define ifb64	long long
	#endif
	#ifndef _WCHAR_T_DEFINED
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif
#else
	#ifndef ifb64
	#define ifb64	__int64
	#endif
	#ifndef _WCHAR_T_DEFINED
	#include	<wchar.h>
	#endif
#endif

#include	<stdio.h>
#include	<string.h>


#define		CFB_BUFFER_SIZE	  0x8000	// Standardpufferlnge
#define		CFB_READ          0			// Nur Lesen
#define		CFB_WRITE         1			// Lesen/Schreiben
#define		CFB_CREATE        2			// Neu erzeugen
#define		CFB_FIXED         3			// Fixer Speicher beim Speichern


typedef		int	  (_cdecl *CFB_PROC_CLOSE)(void *                );
typedef		int	  (_cdecl *CFB_PROC_WRITE)(void *,const void*,int);
typedef		int	  (_cdecl *CFB_PROC_READ )(void *,      void*,int);
typedef		int	  (_cdecl *CFB_PROC_SEEK )(void *,int        ,int);
typedef		int	  (_cdecl *CFB_PROC_CONV )(void *,     void *,int);
typedef		char *(_cdecl *CFB_PROC_BUFF )(void *                );

typedef struct
	{
	CFB_PROC_WRITE	pWrite;
	CFB_PROC_READ	pRead;
	CFB_PROC_SEEK	pSeek;
	CFB_PROC_CONV	pConv;
	}FileBufferConvert;


#ifdef		UNICODE
#define		CFB_ANSITOMODE	(&CFileBuffer::sAnsiToWide)
#define 	CFB_WIDETOMODE	NULL
#define 	CFB_WRITELF		WriteLFW
#else
#define 	CFB_ANSITOMODE	NULL
#define 	CFB_WIDETOMODE	(&CFileBuffer::sWideToAnsi)
#define 	CFB_WRITELF		WriteLFA
#endif

#define 	CFB_ANSITOWIDE	(&CFileBuffer::sWideToAnsi)
#define 	CFB_WIDETOANSI	(&CFileBuffer::sAnsiToWide)
#define 	CFB_LINE_A		((const char*)NULL)
#define 	CFB_LINE_W		((const char*)NULL)


class CFileBuffer
{
//*****************************************************************************
//*
//*     Funktionen
//*
//*****************************************************************************

public:

	static	FileBufferConvert	sWideToAnsi;
	static	FileBufferConvert	sAnsiToWide;


	CFileBuffer(const wchar_t *pName,int iBufferSize=CFB_BUFFER_SIZE);
	CFileBuffer(const char    *pName,int iBufferSize=CFB_BUFFER_SIZE);
	CFileBuffer(                     int iBufferSize=CFB_BUFFER_SIZE);
   ~CFileBuffer();

	bool	Open    (const wchar_t  *pName,int iMode=CFB_READ,int iSeekEnd=0);
	bool	Open    (const char     *pName,int iMode=CFB_READ,int iSeekEnd=0);
	bool	OpenWide(const wchar_t  *pName,int iMode=CFB_READ,int iSeekEnd=0,FileBufferConvert *pConverter=CFB_WIDETOMODE);
	bool	OpenWide(const char     *pName,int iMode=CFB_READ,int iSeekEnd=0,FileBufferConvert *pConverter=CFB_WIDETOMODE);
	bool	OpenAnsi(const wchar_t  *pName,int iMode=CFB_READ,int iSeekEnd=0,FileBufferConvert *pConverter=CFB_ANSITOMODE);
	bool	OpenAnsi(const char     *pName,int iMode=CFB_READ,int iSeekEnd=0,FileBufferConvert *pConverter=CFB_ANSITOMODE);
	bool	OpenConv(const char     *pName,int iMode=CFB_READ,int iSeekEnd=0,FileBufferConvert *pConverter=NULL);
	bool	OpenConv(const wchar_t  *pName,int iMode=CFB_READ,int iSeekEnd=0,FileBufferConvert *pConverter=NULL);
	bool	OpenMem (void *pMem,int iLen=0,int iMode=CFB_READ,               FileBufferConvert *pConverter=NULL);
	bool	OpenMem (                                                        FileBufferConvert *pConverter=NULL);

	bool	OpenRes (const wchar_t  *pName,const wchar_t *pType=0);
	bool	OpenRes (const char     *pName,const char    *pType=0);
	bool	OpenRes (unsigned        pName,const wchar_t *pType=0);
	bool	OpenRes (unsigned        pName,const char    *pType=0);


	bool	OpenProc(void *pHandle,	CFB_PROC_READ  pRead, 			// Allgemeines ffnen
									CFB_PROC_CLOSE pClose=0,
									CFB_PROC_WRITE pWrite=0,
									CFB_PROC_SEEK  pSeek =0);

	bool	Close();												// Datei schliesen
	bool	Flush();					  							// Puffer speichern

	int		Read(unsigned   char  *pMem,int iLen);					// Datei Mem   lesen
	int		Read(           char  *pMem,int iLen);					// Datei Mem   lesen
	int		Read(           void  *pMem,int iLen);					// Datei Mem   lesen
	int		Read(unsigned   int   *iValue);		  					// Datei Zahl  lesen
	int		Read(           int   *iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   long  *iValue);		  					// Datei Zahl  lesen
	int		Read(           long  *iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   short *iValue);		  					// Datei Zahl  lesen
	int		Read(           short *iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   char  *iValue);		  					// Datei Zahl  lesen
	int		Read(           char  *iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   ifb64 *iValue);		  					// Datei Zahl  lesen
	int		Read(           ifb64 *iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   int   &iValue);		  					// Datei Zahl  lesen
	int		Read(           int   &iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   long  &iValue);		  					// Datei Zahl  lesen
	int		Read(           long  &iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   short &iValue);		  					// Datei Zahl  lesen
	int		Read(           short &iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   char  &iValue);		  					// Datei Zahl  lesen
	int		Read(           char  &iValue);		  					// Datei Zahl  lesen
	int		Read(unsigned   ifb64 &iValue);		  					// Datei Zahl  lesen
	int		Read(           ifb64 &iValue);		  					// Datei Zahl  lesen
	int		ReadByte ();					  						// Datei Byte  lesen
	int		ReadWord ();					  						// Datei Word  lesen
	int		ReadDWord();					  						// Datei DWord lesen
    ifb64	ReadQWord();					  						// Datei QWord lesen
	int		ReadLine(unsigned char  *pBuffer,int iLen);				// Eine Zeile einlesen
	int		ReadLine(         char  *pBuffer,int iLen);				// Eine Zeile einlesen
	int		ReadLine(unsigned short *pBuffer,int iLen);				// Eine Zeile einlesen
	int		ReadLine(         short *pBuffer,int iLen);				// Eine Zeile einlesen

	int		Write(const unsigned char  *pMem,int iLen);				// Datei pMem   schreiben
	int		Write(const          char  *pMem,int iLen);				// Datei pMem   schreiben
	int		Write(const          void  *pMem,int iLen);				// Datei pMem   schreiben
	int		Write(const unsigned char  *pMem);						// Datei pMem   schreiben
	int		Write(const          char  *pMem);						// Datei pMem   schreiben
	int		Write(const unsigned short *pMem);						// Datei pMem   schreiben
	int		Write(const          short *pMem);						// Datei pMem   schreiben
	int		Write(unsigned   int   iValue);	  						// Datei Zahl  schreiben
	int		Write(           int   iValue);	  						// Datei Zahl  schreiben
	int		Write(unsigned   short iValue);	  						// Datei Zahl  schreiben
	int		Write(           short iValue);	  						// Datei Zahl  schreiben
	int		Write(unsigned   char  iValue);	  						// Datei Zahl  schreiben
	int		Write(           char  iValue);	  						// Datei Zahl  schreiben
	int		Write(unsigned   ifb64 iValue);	  						// Datei Zahl  schreiben
	int		Write(           ifb64 iValue);	  						// Datei Zahl  schreiben
	int		WriteLF ();						  						// Datei Zeile schreiben
	int		WriteLFA();						  						// Datei Zeile schreiben
	int		WriteLFW();						  						// Datei Zeile schreiben
	int     WriteLF   (const char    *pText);						// Datei Zeile schreiben
	int     WriteLF   (const wchar_t *pText);						// Datei Zeile schreiben
	void    WriteByte (      int   iValue);	  						// Datei Byte  schreiben
	void    WriteWord (      int   iValue);	  						// Datei Word  schreiben
	void    WriteDWord(      int   iValue);	  						// Datei DWord schreiben
	void    WriteQWord(      ifb64 iValue);	  						// Datei QWord schreiben
int	__cdecl	WriteFormat(const char    *pFormat, ...);
int	__cdecl	WriteFormat(const wchar_t *pFormat, ...);


#ifdef _NATIVE_WCHAR_T_DEFINED
	int		Write(      wchar_t  iValue);	 						// Datei Zahl  schreiben
	int		Write(const wchar_t *pMem);								// Datei pMem   schreiben
#endif


	char   *Buffer();												// Zeiger auf Puffer holen
	int		BufferLen(int iLen=0,int iMin=0);	  					// Pufferlnge einstellen
	int		BufferFill(int iMin=0x10000000);						// Pufferlnge nachfllen
	int		BufferFill(int iAdd,void *pMem,int iMin=0x10000000);	// Pufferlnge nachfllen mit berspringen
	bool	Eof();							  						// Datei Ende erreicht
	int		Tell();							  						// Datei Position
	int		Seek(int iPos,int iMode=SEEK_SET);  					// Datei Position setzen
	int		GetLength();											// Dateilnge holen
	int		GetRealLength();										// Tatschliche Lnge holen

	int		GetUserData();											// User Daten lesen
	void   *GetUserDataPtr();
	void	SetUserData(int   iData);								// Userdaten stetzen
	void	SetUserData(void *pData);

protected:

	void	Init(int iBufferSize);									// Initialisieren

	static	char *_cdecl BufferNull(void *pClass);
	static	char *_cdecl BufferNone(void *pClass);
	static	char *_cdecl BufferGet (void *pClass);

	static	int	  _cdecl MemClose  (void *pHandle);
	static	int	  _cdecl MemWrite  (void *pHandle,const void *pBuffer,int iCount);
	static	int	  _cdecl MemRead   (void *pHandle,      void *pBuffer,int iCount);
	static	int	  _cdecl MemSeek   (void *pHandle,int iPos,int iMode);

	static	int	  _cdecl ResClose  (void *pHandle);

	static	char *_cdecl WideBuffer(void *pClass);
	static	int	  _cdecl WideWrite (void *pHandle,const void *pBuffer,int iCount);
	static	int	  _cdecl WideRead  (void *pHandle,      void *pBuffer,int iCount);
	static	int	  _cdecl WideSeek  (void *pHandle,int   iPos ,int iMode);
	static	int	  _cdecl WideConv  (void *pClass ,void *pData,int iMode);

	static	char *_cdecl AnsiBuffer(void *pClass);
	static	int	  _cdecl AnsiClose (void *pHandle);
	static	int	  _cdecl AnsiWrite (void *pHandle,const void *pBuffer,int iCount);
	static	int	  _cdecl AnsiRead  (void *pHandle,      void *pBuffer,int iCount);
	static	int	  _cdecl AnsiSeek  (void *pHandle,int   iPos ,int iMode);
	static	int	  _cdecl AnsiConv  (void *pClass ,void *pData,int iMode);


//*****************************************************************************
//*
//*     Variabeln
//*
//*****************************************************************************
protected:

	void		   *pFileHandle;				// Dateihandle
	int				iOpenMode;					// ffnungs Modus 0=Lesen 1=Schreiben
	bool			bEofFlag;					// Ende der Datei erreicht
	bool			bMemFlag;					// Keine Datei nur Speicher
	bool			bFixFlag;					// Speicher hat fixe Lnge
	bool			bStopFlag;					// Nichts mehr einlesen
	bool			bOpenFlag;					// Ist etwas geffnet
	bool			bWriteFlag;					// Puffer schreiben
												
	int				iBufferPos;					// Puffer Lese/Schreib Position
	int				iBufferEnd;					// Puffer Byte-Anzahl
	int				iBufferLen;					// Puffer Gre gesamt
	int				iBufferOff;					// Puffer Offset im Stream
	int				iBufferMin;					// Puffer minimal Byteanzahl
	char		   *pBuffer;					// Zeiger auf Pufferspeicher
				
	void		   *pUserData;					// Benutzer Daten
	void		   *hResHg;						// Resource HG Handel
	void		   *hResSrc;					// Resource Handel

	CFB_PROC_CLOSE	pProcClose;					// Dateifunktionen
	CFB_PROC_WRITE	pProcWrite;					//		*
	CFB_PROC_READ	pProcRead;					//		*
	CFB_PROC_SEEK	pProcSeek;					//		*
	CFB_PROC_CONV 	pProcConv;					//		*
	CFB_PROC_BUFF	pProcBuff;					//		*

private:

	void		   *pOldHandle;					// Altes Datei-Handle
	char		   *pOldBuffer;					// Zeiger auf altenPufferspeicher
	int				iCloseSeek;					// Dateiposition fr's schlieen
	int				iConvLen;					// Position bis zu der
	int				iConvPos;					// Position bis zu der
	int				iSeekAdd;					// Letzte Schreibposition
	int 			iCheckWr;					// Letztes Schreibeoperation prfen

	CFB_PROC_WRITE	pOldWrite;					//		*
	CFB_PROC_READ	pOldRead;					//		*
	CFB_PROC_SEEK	pOldSeek;					//		*
};

//*****************************************************************************
//*
//*    Inline Funktionen
//*
//*****************************************************************************
#ifndef _DEBUG

inline bool  CFileBuffer::Eof(){return bEofFlag;}

inline int   CFileBuffer::Write   (const unsigned char  *pMem,int iLen){return Write((const char*)pMem,iLen);}
inline int   CFileBuffer::Write   (const          void  *pMem,int iLen){return Write((const char*)pMem,iLen);}
inline int   CFileBuffer::Write   (const unsigned char  *pMem         ){return Write((const char*)pMem,(int)strlen((const char*)pMem));}
inline int   CFileBuffer::Write   (const          char  *pMem         ){return Write((const char*)pMem,(int)strlen((const char*)pMem));}
inline int   CFileBuffer::Write   (const unsigned short *pMem         ){return Write((const char*)pMem,(int)wcslen((const wchar_t*)pMem)*2);}
inline int   CFileBuffer::Write   (const          short *pMem         ){return Write((const char*)pMem,(int)wcslen((const wchar_t*)pMem)*2);}
inline int   CFileBuffer::Write   (unsigned       ifb64 iValue){return Write((char*)&iValue,sizeof(unsigned   ifb64));}
inline int   CFileBuffer::Write   (               ifb64 iValue){return Write((char*)&iValue,sizeof(           ifb64));}
inline int   CFileBuffer::Write   (unsigned       int   iValue){return Write((char*)&iValue,sizeof(unsigned   int  ));}
inline int   CFileBuffer::Write   (               int   iValue){return Write((char*)&iValue,sizeof(           int  ));}
inline int   CFileBuffer::Write   (unsigned       short iValue){return Write((char*)&iValue,sizeof(unsigned   short));}
inline int   CFileBuffer::Write   (               short iValue){return Write((char*)&iValue,sizeof(           short));}
inline int   CFileBuffer::Write   (unsigned       char  iValue){return Write((char*)&iValue,sizeof(unsigned   char ));}
inline int   CFileBuffer::Write   (               char  iValue){return Write((char*)&iValue,sizeof(           char ));}
inline int   CFileBuffer::WriteLFA(                           ){return Write( "\r\n",2);}
inline int   CFileBuffer::WriteLFW(                           ){return Write(L"\r\n",4);}
inline int   CFileBuffer::WriteLF (const char           *pText){return Write( "\r\n",2);}
inline int   CFileBuffer::WriteLF (const wchar_t        *pText){return Write(L"\r\n",4);}
inline int   CFileBuffer::WriteLF (                           ){return CFB_WRITELF()   ;}

#ifdef _NATIVE_WCHAR_T_DEFINED
inline int	CFileBuffer::Write(      wchar_t  iValue){return Write((char*)&iValue,sizeof(wchar_t));}
inline int	CFileBuffer::Write(const wchar_t *pMem  ){return Write((const char*)pMem,(int)wcslen(pMem)*2);}
#endif

inline int   CFileBuffer::Read(unsigned       char  *pMem,int iLen){return Read((char*)pMem,iLen);};
inline int   CFileBuffer::Read(			      void  *pMem,int iLen){return Read((char*)pMem,iLen);};
inline int   CFileBuffer::Read(unsigned       ifb64 *pValue){return Read((char*) pValue,sizeof(unsigned   ifb64));}
inline int   CFileBuffer::Read(               ifb64 *pValue){return Read((char*) pValue,sizeof(           ifb64));}
inline int   CFileBuffer::Read(unsigned       int   *pValue){return Read((char*) pValue,sizeof(unsigned   int  ));}
inline int   CFileBuffer::Read(               int   *pValue){return Read((char*) pValue,sizeof(           int  ));}
inline int   CFileBuffer::Read(unsigned       long  *pValue){return Read((char*) pValue,sizeof(unsigned   long ));}
inline int   CFileBuffer::Read(               long  *pValue){return Read((char*) pValue,sizeof(           long ));}
inline int   CFileBuffer::Read(unsigned       short *pValue){return Read((char*) pValue,sizeof(unsigned   short));}
inline int   CFileBuffer::Read(               short *pValue){return Read((char*) pValue,sizeof(           short));}
inline int   CFileBuffer::Read(unsigned       char  *pValue){return Read((char*) pValue,sizeof(unsigned   char ));}
inline int   CFileBuffer::Read(               char  *pValue){return Read((char*) pValue,sizeof(           char ));}
inline int   CFileBuffer::Read(unsigned       ifb64 &iValue){return Read((char*)&iValue,sizeof(unsigned   ifb64));}
inline int   CFileBuffer::Read(               ifb64 &iValue){return Read((char*)&iValue,sizeof(           ifb64));}
inline int   CFileBuffer::Read(unsigned       int   &iValue){return Read((char*)&iValue,sizeof(unsigned   int  ));}
inline int   CFileBuffer::Read(               int   &iValue){return Read((char*)&iValue,sizeof(           int  ));}
inline int   CFileBuffer::Read(unsigned       long  &iValue){return Read((char*)&iValue,sizeof(unsigned   long ));}
inline int   CFileBuffer::Read(               long  &iValue){return Read((char*)&iValue,sizeof(           long ));}
inline int   CFileBuffer::Read(unsigned       short &iValue){return Read((char*)&iValue,sizeof(unsigned   short));}
inline int   CFileBuffer::Read(               short &iValue){return Read((char*)&iValue,sizeof(           short));}
inline int   CFileBuffer::Read(unsigned       char  &iValue){return Read((char*)&iValue,sizeof(unsigned   char ));}
inline int   CFileBuffer::Read(               char  &iValue){return Read((char*)&iValue,sizeof(           char ));}
inline int	 CFileBuffer::ReadLine(unsigned char  *pMem,int iLen){return ReadLine((char          *)pMem,iLen);}
inline int	 CFileBuffer::ReadLine(         short *pMem,int iLen){return ReadLine((unsigned short*)pMem,iLen);}

inline char *CFileBuffer::Buffer(               ){return  pProcBuff(this);}
inline void *CFileBuffer::GetUserDataPtr(       ){return        pUserData;}
inline int	 CFileBuffer::GetUserData(          ){return   (int)pUserData;}
inline void	 CFileBuffer::SetUserData(int   iData){pUserData=(void*)iData;}
inline void	 CFileBuffer::SetUserData(void *pData){pUserData=       pData;}

inline bool	 CFileBuffer::OpenWide(const wchar_t *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
inline bool	 CFileBuffer::OpenWide(const char    *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
inline bool	 CFileBuffer::OpenAnsi(const wchar_t *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
inline bool	 CFileBuffer::OpenAnsi(const char    *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
inline bool  CFileBuffer::OpenMem (                                            FileBufferConvert *pConverter){return OpenMem(0,0,CFB_CREATE       ,pConverter);}




#endif
#endif
