//*****************************************************************************
//*
//*
//*		HttpMd5.cpp
//*
//*
//*****************************************************************************
#include	<memory.h>
#include	"HttpMd5.h"


void Md5Transform(umd5_32 *pBuffer,umd5_32 *pInput);


//*****************************************************************************
//*
//*		ByteReverse
//*
//*****************************************************************************
#ifndef		HIGHFIRST					
#define		ByteReverse(pBuffer,uSize)
#else
static void ByteReverse(void *pData,unsigned uSize)
{
unsigned char  *pBuffer=(unsigned char*)pData;
umd5_32		    uVal;


	do	{	
		uVal = (umd5_32) ((unsigned) pBuffer[3]<<8 | pBuffer[2]) << 16 |
		                 ((unsigned) pBuffer[1]<<8 | pBuffer[0]);
		
		*(umd5_32*) pBuffer = uVal;
		pBuffer += 4;
		uSize   -= 1;
		}while(uSize);

}
#endif


//*****************************************************************************
//*
//*		Md5Init
//*
//*****************************************************************************
// 	Start Md5 accumulation.	 Set bit count to 0 and buffer to mysterious
// 	initialization constants.
void Md5Init(Md5Context *pCtx)
{

	pCtx->aBuffer [0] = 0x67452301;
	pCtx->aBuffer [1] = 0xEFCDAB89;
	pCtx->aBuffer [2] = 0x98BADCFE;
	pCtx->aBuffer [3] = 0x10325476;
	pCtx->aBits   [0] = 0;
	pCtx->aBits   [1] = 0;

}

//*****************************************************************************
//*
//*		Md5Update
//*
//*****************************************************************************
// 	Update context to reflect the concatenation of another buffer full of bytes.
void Md5Update(Md5Context *pCtx,const void *pInput,unsigned uSize)
{
const unsigned char *pBuffer=(unsigned char*)pInput;
unsigned char       *pPtr;
umd5_32				 uVal;


																	

	uVal			= pCtx->aBits[0];								// Update bitcount
	pCtx->aBits[0]	= uVal + ((umd5_32) uSize << 3);
	
	if(pCtx->aBits[0] < uVal)
		{
		pCtx->aBits[1]++;											// Carry from low to high
		}

	pCtx->aBits[1] += uSize >> 29;

	uVal = (uVal >> 3) & 0x3F;										// Bytes already in shsInfo->data

	

	if(uVal)														// Handle any leading odd-sized chunks
		{
		pPtr = pCtx->aInput + uVal;
		uVal = 64 - uVal;

		if(uSize < uVal)
			{
			memcpy(pPtr, pBuffer, uSize);
			return;
			}

		memcpy(pPtr,pBuffer,uVal);
		ByteReverse (pCtx->aInput ,16);
		Md5Transform(pCtx->aBuffer,(umd5_32*) pCtx->aInput);
		pBuffer += uVal;
		uSize   -= uVal;
		}
	

	while(uSize>=64)												// Process data in 64-byte chunks
		{
		memcpy(pCtx->aInput,pBuffer,64);
		ByteReverse (pCtx->aInput , 16);
		Md5Transform(pCtx->aBuffer, (umd5_32*) pCtx->aInput);
		pBuffer += 64;
		uSize   -= 64;
		}


	memcpy(pCtx->aInput,pBuffer,uSize);								// Handle any remaining bytes of data.

}

//*****************************************************************************
//*
//*		Md5Final
//*
//*****************************************************************************
// 	Final wrapup - pad to 64-byte boundary with the bit pattern
// 	1 0* (64-bit uCount of aBits processed, MSB-first)
void Md5Final(Md5Context *pCtx,void  *pData)
{
unsigned char  *pOutput=(unsigned char*)pData;
unsigned char  *pPtr;
unsigned		uCnt;

	
	
	uCnt    = (pCtx->aBits[0]>>3)&0x3F;								// Compute number of bytes mod 64
	pPtr    = pCtx->aInput + uCnt;									// Set the first char of padding to 0x80.  This is safe since there is always at least one byte free 
	pPtr[0] = 0x80;
	pPtr   += 1;
	uCnt    = 64-1-uCnt;											// Bytes of padding needed to make 64 bytes

	if(uCnt<8)														// Pad out to 56 mod 64
		{
		memset(pPtr,0,uCnt);										// Two lots of padding:	 Pad the first block to 64 bytes
		ByteReverse (pCtx->aInput ,16);
		Md5Transform(pCtx->aBuffer,(umd5_32*)pCtx->aInput);
		memset(pCtx->aInput,0,56);									// Now fill the next block with 56 bytes
		}
	else{
		memset(pPtr,0,uCnt-8);										// Pad block to 56 bytes
		}

	
	ByteReverse(pCtx->aInput,14);
	
	((umd5_32*)pCtx->aInput)[14] = pCtx->aBits[0];					// Append length in bits and transform
	((umd5_32*)pCtx->aInput)[15] = pCtx->aBits[1];

	Md5Transform(pCtx->aBuffer, (umd5_32*) pCtx->aInput);
	ByteReverse (pCtx->aBuffer,4);
	memcpy(pOutput,pCtx->aBuffer,16);
	memset(pCtx,0,sizeof(pCtx));									// In case it's sensitive
}



//*****************************************************************************
//*
//*		Md5Transform
//*
//*****************************************************************************
// 	The core of the Md5 algorithm, this alters an existing Md5 hash to
// 	reflect the addition of 16 longwords of new data.  Md5Update blocks
// 	the data and converts bytes into longwords for this routine.
void Md5Transform(umd5_32 *pBuffer,umd5_32 *pInput)
{
register umd5_32		a, b, c, d;


	// The four core functions - F1 is optimized somewhat
	#define		F1(x,y,z)				(z ^ (x & (y ^ z)))
	#define		F2(x,y,z)				F1(z, x, y)
	#define		F3(x,y,z)				(x ^ y ^ z)
	#define		F4(x,y,z)				(y ^ (x | ~z))

	// This is the central step aInput the Md5 algorithm.
	#define		Md5STEP(f,w,x,y,z,data,s)	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )


	a = pBuffer[0];
	b = pBuffer[1];
	c = pBuffer[2];
	d = pBuffer[3];

	Md5STEP(F1,a,b,c,d,pInput[ 0] + 0xD76AA478 , 7);
	Md5STEP(F1,d,a,b,c,pInput[ 1] + 0xE8C7B756 ,12);
	Md5STEP(F1,c,d,a,b,pInput[ 2] + 0x242070DB ,17);
	Md5STEP(F1,b,c,d,a,pInput[ 3] + 0xC1BDCEEE ,22);
	Md5STEP(F1,a,b,c,d,pInput[ 4] + 0xF57C0FAF , 7);
	Md5STEP(F1,d,a,b,c,pInput[ 5] + 0x4787C62A ,12);
	Md5STEP(F1,c,d,a,b,pInput[ 6] + 0xA8304613 ,17);
	Md5STEP(F1,b,c,d,a,pInput[ 7] + 0xFD469501 ,22);
	Md5STEP(F1,a,b,c,d,pInput[ 8] + 0x698098D8 , 7);
	Md5STEP(F1,d,a,b,c,pInput[ 9] + 0x8B44F7AF ,12);
	Md5STEP(F1,c,d,a,b,pInput[10] + 0xFFFF5BB1 ,17);
	Md5STEP(F1,b,c,d,a,pInput[11] + 0x895CD7BE ,22);
	Md5STEP(F1,a,b,c,d,pInput[12] + 0x6B901122,  7);
	Md5STEP(F1,d,a,b,c,pInput[13] + 0xFD987193 ,12);
	Md5STEP(F1,c,d,a,b,pInput[14] + 0xA679438E ,17);
	Md5STEP(F1,b,c,d,a,pInput[15] + 0x49B40821 ,22);

	Md5STEP(F2,a,b,c,d,pInput[ 1] + 0xF61E2562 , 5);
	Md5STEP(F2,d,a,b,c,pInput[ 6] + 0xC040B340 , 9);
	Md5STEP(F2,c,d,a,b,pInput[11] + 0x265E5A51 ,14);
	Md5STEP(F2,b,c,d,a,pInput[ 0] + 0xE9B6C7AA ,20);
	Md5STEP(F2,a,b,c,d,pInput[ 5] + 0xD62F105D , 5);
	Md5STEP(F2,d,a,b,c,pInput[10] + 0x02441453 , 9);
	Md5STEP(F2,c,d,a,b,pInput[15] + 0xD8A1E681 ,14);
	Md5STEP(F2,b,c,d,a,pInput[ 4] + 0xE7D3FBC8 ,20);
	Md5STEP(F2,a,b,c,d,pInput[ 9] + 0x21E1CDE6 , 5);
	Md5STEP(F2,d,a,b,c,pInput[14] + 0xC33707D6 , 9);
	Md5STEP(F2,c,d,a,b,pInput[ 3] + 0xF4D50D87 ,14);
	Md5STEP(F2,b,c,d,a,pInput[ 8] + 0x455A14ED ,20);
	Md5STEP(F2,a,b,c,d,pInput[13] + 0xA9E3E905 , 5);
	Md5STEP(F2,d,a,b,c,pInput[ 2] + 0xFCEFA3F8 , 9);
	Md5STEP(F2,c,d,a,b,pInput[ 7] + 0x676F02D9 ,14);
	Md5STEP(F2,b,c,d,a,pInput[12] + 0x8D2A4C8A ,20);

	Md5STEP(F3,a,b,c,d,pInput[ 5] + 0xFFFA3942 , 4);
	Md5STEP(F3,d,a,b,c,pInput[ 8] + 0x8771F681 ,11);
	Md5STEP(F3,c,d,a,b,pInput[11] + 0x6D9D6122 ,16);
	Md5STEP(F3,b,c,d,a,pInput[14] + 0xFDE5380C ,23);
	Md5STEP(F3,a,b,c,d,pInput[ 1] + 0xA4BEEA44 , 4);
	Md5STEP(F3,d,a,b,c,pInput[ 4] + 0x4BDECFA9 ,11);
	Md5STEP(F3,c,d,a,b,pInput[ 7] + 0xF6BB4B60 ,16);
	Md5STEP(F3,b,c,d,a,pInput[10] + 0xBEBFBC70 ,23);
	Md5STEP(F3,a,b,c,d,pInput[13] + 0x289B7EC6 , 4);
	Md5STEP(F3,d,a,b,c,pInput[ 0] + 0xEAA127FA ,11);
	Md5STEP(F3,c,d,a,b,pInput[ 3] + 0xD4EF3085 ,16);
	Md5STEP(F3,b,c,d,a,pInput[ 6] + 0x04881D05 ,23);
	Md5STEP(F3,a,b,c,d,pInput[ 9] + 0xD9D4D039 , 4);
	Md5STEP(F3,d,a,b,c,pInput[12] + 0xE6DB99E5 ,11);
	Md5STEP(F3,c,d,a,b,pInput[15] + 0x1FA27CF8 ,16);
	Md5STEP(F3,b,c,d,a,pInput[ 2] + 0xC4AC5665 ,23);

	Md5STEP(F4,a,b,c,d,pInput[ 0] + 0xF4292244 , 6);
	Md5STEP(F4,d,a,b,c,pInput[ 7] + 0x432AFF97 ,10);
	Md5STEP(F4,c,d,a,b,pInput[14] + 0xAB9423A7 ,15);
	Md5STEP(F4,b,c,d,a,pInput[ 5] + 0xFC93A039 ,21);
	Md5STEP(F4,a,b,c,d,pInput[12] + 0x655B59C3 , 6);
	Md5STEP(F4,d,a,b,c,pInput[ 3] + 0x8F0CCC92 ,10);
	Md5STEP(F4,c,d,a,b,pInput[10] + 0xFFEFF47D ,15);
	Md5STEP(F4,b,c,d,a,pInput[ 1] + 0x85845DD1 ,21);
	Md5STEP(F4,a,b,c,d,pInput[ 8] + 0x6FA87E4F , 6);
	Md5STEP(F4,d,a,b,c,pInput[15] + 0xFE2CE6E0 ,10);
	Md5STEP(F4,c,d,a,b,pInput[ 6] + 0xA3014314 ,15);
	Md5STEP(F4,b,c,d,a,pInput[13] + 0x4E0811A1 ,21);
	Md5STEP(F4,a,b,c,d,pInput[ 4] + 0xF7537E82 , 6);
	Md5STEP(F4,d,a,b,c,pInput[11] + 0xBD3AF235 ,10);
	Md5STEP(F4,c,d,a,b,pInput[ 2] + 0x2AD7D2BB ,15);
	Md5STEP(F4,b,c,d,a,pInput[ 9] + 0xEB86D391 ,21);

	pBuffer[0] += a;
	pBuffer[1] += b;
	pBuffer[2] += c;
	pBuffer[3] += d;
}

//*****************************************************************************
//*
//*		Md5ToString
//*
//*****************************************************************************
void Md5ToString(char *pStr,const void *pInput)
{
unsigned char  *pData=(unsigned char*)pInput; 
int				iPos;


	for(iPos=0;iPos<16;iPos++)
		{
		pStr[iPos*2  ] = "0123456789abcdef"[pData[iPos]>>  4];
		pStr[iPos*2+1] = "0123456789abcdef"[pData[iPos]&0x0F];
		}

	pStr[32]=0;

}
