//****************************************************************************
//*
//*
//*		 MapFile.cpp
//*
//*
//****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Die Klasse CMapFile erzeugt bzw. ffnet ein MemoryMappedFile
//	Die Klasse ist fr UNICODE und ANSI geeignet.
//
//	Funktionen:		char *Open(...)		Datei ffnen
//					char *Map (...)		Mapping einstellen
//					char *GetMem()		Speicherzeiger
//					int	  GetLength()	Speicherlnge
//					int	  Close()		Datei schliesen
//					int	  Unmap()		Mapping aufheben
//
#if defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif
#include	<Afx.h>
#else
#include    <Windows.h>
#endif

#include	"MapFile.h"


#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


static	int		iPageSize=0;									// Gre einer Speichersiete

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CMapFile::CMapFile()
{

	iFileLength = 0;											// Lnge des Files
	pObjName	= 0;											// Zeiger auf Objekt Namem
	pMapMem 	= 0;											// Zeiger auf Speicher
	iMapLen 	= 0;											// Lnge des Speichers
	hMapObj 	= 0;											// Memorymaped Objekt
	hMapFile	= INVALID_HANDLE_VALUE;							// Memorymaped File

}

//*****************************************************************************
//*
//*		CMapFile
//*
//*****************************************************************************
CMapFile::CMapFile(const char *pFilename,int iMode,const char *pName,unsigned uSize)
{
	CMapFile();
	Open(pFilename,iMode,pName,uSize);
}

CMapFile::CMapFile(const wchar_t *pFilename,int iMode,const wchar_t *pName,unsigned uSize)
{
	CMapFile();
	Open(pFilename,iMode,pName,uSize);
}

//*****************************************************************************
//*
//*		CMapFile
//*
//*****************************************************************************
CMapFile::CMapFile(int iMode,const char *pName,int iLen)
{
	CMapFile();
	Open(iMode,pName,iLen);
}

CMapFile::CMapFile(int iMode,const wchar_t *pName,int iLen)
{
	CMapFile();
	Open(iMode,pName,iLen);
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CMapFile::~CMapFile()
{
	if(hMapObj || pObjName)Close();
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
// 	Schliet das Memmory-Mapped-File
// 	Ergibt 1 wenn File geschlossen wurde bzw. 0 bei einem Fehler
int CMapFile::Close()
{
BOOL	bExit=1;



	if(pObjName)												// Objekt Namem freigeben
		{
		delete pObjName;
		pObjName = 0;
		}

	if(hMapObj!=0)												// Objekt schlieen
	if(hMapObj!=INVALID_HANDLE_VALUE)
		{
		if(!Unmap())bExit=0;
		if(!CloseHandle(hMapObj))bExit=0;
		}

	if(hMapFile!=INVALID_HANDLE_VALUE)							// File schlieen
		{
		if(!CloseHandle(hMapFile))bExit=0;
		hMapFile=INVALID_HANDLE_VALUE;
		}

	iFileLength = 0;											// Lnge des Files
	pMapMem 	= 0;											// Zeiger auf Speicher
	iMapLen 	= 0;											// Lnge des Speichers
	hMapObj 	= 0;


return bExit;
}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
// 	ffnet ein vorhandes Memmory-Mapped-Objekt
//	iMode		: Bit0 !=0 so ist auch Schreiben erlaubt.
// 				  Bit1 !=0 so wird das Objekt nicht gemappet und 0 zurckgegeben.
// 				  Bit4 !=0 so wird ein temorres Speicherobjekt erzeugt.
//	pName		: Ist der Name des Memmory-Mapped-Objekt
//	iMemLen		: Ist die Gre in Byte mit der das Memmory-Mapped-Objekt
//				  erzeugt wird.	(Dazu muss Bit4 in iMode gesetzt sein)
// 	Ergibt einen Zeiger auf den Speicher-Bereich bzw 0 bei einem Fehler
char *CMapFile::Open(int iMode,const char *pName,int iMemLen)
{
SECURITY_ATTRIBUTES sSecAttr;
int					iFlags;
int					iLen;
int					i;



	Close();

	if(pName)										// Objektnamen speichern
		{
		for(iLen=0;pName[iLen];iLen++);
		iLen++;

		pObjName = new wchar_t[iLen];

		for(i=0;i<iLen;i++)							// Namen kopieren
			{
			pObjName[i] = (unsigned char)pName[i];
			}
		}


	if(iMemLen<=0)
			iFileLength = 0;
	else	iFileLength = iMemLen;


	iOpenMode = iMode;

//******************** Objekt ffnen ******************************************

	if(iMode&0x10)									// Temporres Objekt erzeugen
		{
		iFlags = (iMode&1)? PAGE_READWRITE:PAGE_READONLY;

		sSecAttr.nLength				=  sizeof(sSecAttr);
		sSecAttr.lpSecurityDescriptor	=  0;
		sSecAttr.bInheritHandle 		= (pName)? TRUE:FALSE;

		hMapObj = CreateFileMappingA(INVALID_HANDLE_VALUE,&sSecAttr,iFlags,0,iFileLength,pName);
		}
	else{											// Ein vorhandenes Objekt ffnen
		iFlags	= (iMode&1)? FILE_MAP_ALL_ACCESS:FILE_MAP_READ;
		hMapObj =  OpenFileMappingA(iFlags,TRUE,pName);
		}


//**************** Speicherzeiger holen ***************************************

	if(!hMapObj)									// Konnte ein Objekt erzeut werden
		{
		Close();
		return 0;
		}

	if(!(iMode&2))									// Speicher mappen
		{
		Map(0,0);
		if(!pMapMem){Close();return 0;}
		}

	iOpenMode|=0x80;




return pMapMem;
}

char *CMapFile::Open(int iMode,const wchar_t *pName,int iMemLen)
{
SECURITY_ATTRIBUTES sSecAttr;
int					iFlags;
int					iLen;



	Close();

	if(pName)										// Objektnamen speichern
		{
		for(iLen=0;pName[iLen];iLen++);
		iLen++;
		pObjName = new wchar_t[iLen];

		memcpy(pObjName,pName,iLen*sizeof(wchar_t));
		}


	if(iMemLen<=0)
			iFileLength = 0;
	else	iFileLength = iMemLen;


	iOpenMode = iMode;

//******************** Objekt ffnen ******************************************

	if(iMode&0x10)									// Temporres Objekt erzeugen
		{
		iFlags = (iMode&1)? PAGE_READWRITE:PAGE_READONLY;

		sSecAttr.nLength				=  sizeof(sSecAttr);
		sSecAttr.lpSecurityDescriptor	=  0;
		sSecAttr.bInheritHandle 		= (pName)? TRUE:FALSE;

		hMapObj = CreateFileMappingW(INVALID_HANDLE_VALUE,&sSecAttr,iFlags,0,iFileLength,pName);
		}
	else{											// Ein vorhandenes Objekt ffnen
		iFlags	= (iMode&1)? FILE_MAP_ALL_ACCESS:FILE_MAP_READ;
		hMapObj =  OpenFileMappingW(iFlags,TRUE,pName);
		}


//**************** Speicherzeiger holen ***************************************

	if(!hMapObj)									// Konnte eine Objekt erzeut werden
		{
		Close();
		return 0;
		}

	if(!(iMode&2))									// Speicher mappen
		{
		Map(0,0);
		if(!pMapMem){Close();return 0;}
		}

	iOpenMode|=0x80;




return pMapMem;
}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
// 	ffnet das eine Datei als Memmory-Mapped-File
//	pFilename	: Ist der Name der Datei
// 	iMode		: Bit0 !=0 so ist auch Schreiben erlaubt.
// 				  Bit1 !=0 so wird die Datei nicht in den Speicher gemapped
// 				  Bit2 !=0 so ist fr das Objekt mehrfach Schreiben erlaubt
// 				  Bit3 !=0 so ist fr das Objekt kein mehrfach Zugriff erlaubt
// 				  Bit4 !=0 so ein temorres Speicherobjekt erzeugt pName ist
// 						  	dann der pName des Objektes, oder die Datei erzeugen
// 				  Bit5 !=0 So wird die Datei erzeugt und auf 0 gesetzt
// 	Ergibt einen Zeiger auf den Speicher-Bereich bzw 0 bei einem Fehler
char *CMapFile::Open(const char *pFilename,int iMode,const char *pName,unsigned uSize)
{
SECURITY_ATTRIBUTES sSecAttr;
int					iShare;
int					iFlags;
int					iOpen;
int					iLen;
int					i;



	Close();

	if(pName)										// Objektnamen speichern
		{
		for(iLen=0;pName[iLen];iLen++);
		iLen++;
		pObjName = new wchar_t[iLen];

		for(i=0;i<iLen;i++)							// Namen kopieren
			{
			pObjName[i]=(unsigned char)pName[i];
			}
		}


	iOpenMode	=  iMode;
	iFlags		= (iMode&0x01)? GENERIC_READ|GENERIC_WRITE:GENERIC_READ;
	iShare		= (iMode&0x04)? FILE_SHARE_WRITE|FILE_SHARE_READ:FILE_SHARE_READ;


	if(iMode&0x08)iShare=0;

	if(iMode&0x20)
			iOpen=CREATE_ALWAYS;
	else	iOpen=(iMode&0x10)? OPEN_ALWAYS:OPEN_EXISTING;


//******************** Datei ffnen *******************************************


	   hMapFile=CreateFileA(pFilename,iFlags,iShare,0,iOpen,FILE_ATTRIBUTE_NORMAL,0);
	if(hMapFile==INVALID_HANDLE_VALUE)return 0;

	iFileLength = (uSize)? uSize:GetFileSize(hMapFile,0);


//******************** Objekt ffnen ******************************************

	sSecAttr.nLength				=  sizeof(sSecAttr);
	sSecAttr.lpSecurityDescriptor	=  0;
	sSecAttr.bInheritHandle 		= (pName)? TRUE:FALSE;

	if(iFileLength==0 && iMode&0x30)
		{
		hMapObj=INVALID_HANDLE_VALUE;
		return 0;
		}


		hMapObj=CreateFileMappingA(hMapFile,&sSecAttr,(iMode&1)? PAGE_READWRITE:PAGE_READONLY,0,iFileLength,pName);
	if(!hMapObj){Close();return 0;}


//******************** Speicher Zeiger ****************************************

	if(!(iMode&2))
		{
		Map(0,0);

		if(!pMapMem)
			{
			Close();
			return 0;
			}
		}



return pMapMem;
}

char *CMapFile::Open(const wchar_t *pFilename,int iMode,const wchar_t *pName,unsigned uSize)
{
SECURITY_ATTRIBUTES sSecAttr;
int					iFlags;
int					iShare;
int					iOpen;
int					iLen;



	Close();

	if(pName)										// Objektnamen speichern
		{
		for(iLen=0;pName[iLen];iLen++);
		iLen++;
		pObjName = new wchar_t[iLen];

		memcpy(pObjName,pName,iLen*sizeof(wchar_t));
		}


	iOpenMode	=  iMode;
	iFlags		= (iMode&0x01)? GENERIC_READ|GENERIC_WRITE:GENERIC_READ;
	iShare		= (iMode&0x04)? FILE_SHARE_WRITE|FILE_SHARE_READ:FILE_SHARE_READ;


	if(iMode&0x08)iShare=0;

	if(iMode&0x20)
			iOpen=CREATE_ALWAYS;
	else	iOpen=(iMode&0x10)? OPEN_ALWAYS:OPEN_EXISTING;


//******************** Datei ffnen *******************************************


	   hMapFile=CreateFileW(pFilename,iFlags,iShare,0,iOpen,FILE_ATTRIBUTE_NORMAL,0);
	if(hMapFile==INVALID_HANDLE_VALUE)return 0;

	iFileLength = (uSize)? uSize:GetFileSize(hMapFile,0);


//******************** Objekt ffnen ******************************************

	sSecAttr.nLength				=  sizeof(sSecAttr);
	sSecAttr.lpSecurityDescriptor	=  0;
	sSecAttr.bInheritHandle 		= (pName)? TRUE:FALSE;

	if(iFileLength==0 && iMode&0x30)
		{
		hMapObj=INVALID_HANDLE_VALUE;
		return 0;
		}


		hMapObj=CreateFileMappingW(hMapFile,&sSecAttr,(iMode&1)? PAGE_READWRITE:PAGE_READONLY,0,iFileLength,pName);
	if(!hMapObj){Close();return 0;}


//******************** Speicher Zeiger ****************************************

	if(!(iMode&2))
		{
		Map(0,0);

		if(!pMapMem)
			{
			Close();
			return 0;
			}
		}



return pMapMem;
}

//*****************************************************************************
//*
//*		Map
//*
//*****************************************************************************
// 	Mapped ein File
// 	iOffset	: Ist das Offset in der Datei ab der der Speicher eingeblendet wird
// 	iLen	: Ist die Lnge des einzublendenten Speicherbereiches
// 			  Wenn iLen Null ist wird die gesammte Datei eingeblendet.
// 	Ergibt einen Zeiger auf den Speicherbereich bzw 0 bei einem Fehler
char *CMapFile::Map(int iOffset,int iLen)
{
SECURITY_ATTRIBUTES		sSecAttr;
SYSTEM_INFO				sSysInfo;
HANDLE					hObj;
int						iSub;



	if(pMapMem)											// Ist bereits ein Speicher gemapped
		{
		if(iLen==0)iLen=iFileLength;

		if(iLen==iMapLen && iOffset==iMapOff)
			{
			return pMapMem;
			}

		Unmap();
		}


	if(!iPageSize)
		{
		GetSystemInfo(&sSysInfo);
		iPageSize=sSysInfo.dwPageSize;
		}

	if(hMapObj==0)return 0;
	if(hMapObj==INVALID_HANDLE_VALUE)					// Ein neues Objekt erzeugen ?
		{
		if(iLen<=0)return 0;
		if(!(iOpenMode&0x31))return 0;

		sSecAttr.nLength				=  sizeof(sSecAttr);
		sSecAttr.lpSecurityDescriptor	=  0;
		sSecAttr.bInheritHandle 		= (pObjName)? TRUE:FALSE;

			hObj=CreateFileMappingW(hMapFile,&sSecAttr,(iOpenMode&1)? PAGE_READWRITE:PAGE_READONLY,0,iLen,pObjName);
		if(!hObj)return 0;
		if(iFileLength<iLen)iFileLength=iLen;
		hMapObj=hObj;
		}


	iSub    =  iOffset&(iPageSize-1);
	iMapLen =  0;
	pMapMem = (char*)MapViewOfFile(hMapObj,(iOpenMode&1)? FILE_MAP_WRITE:FILE_MAP_READ,iOffset-iSub,0,iLen+iSub);

	if(pMapMem)
		{
		iMapOff  =  iOffset;
		iMapLen  = (iLen)? iLen:iFileLength-iOffset;
		pMapMem +=  iSub;
		}


return pMapMem;
}

//*****************************************************************************
//*
//*		SetSize
//*
//*****************************************************************************
// 	ndert die Gre eines Memory-Map-File ein.
//	Dabei wird die der Speicher
// 	Ergibt den neuen Zeiger auf den Speicherbereich
char *CMapFile::SetSize(int iLen)
{


	if(hMapObj==0)return 0;
	if(!(iOpenMode&1))return 0;
	if(iLen<=0)return 0;

	Unmap();

	if(hMapObj!=INVALID_HANDLE_VALUE)
		{
		CloseHandle(hMapObj);
		hMapObj=INVALID_HANDLE_VALUE;
		}


return Map(0,iLen);
}

//*****************************************************************************
//*
//*		Unmap
//*
//*****************************************************************************
// 	Hebt das Mapping auf, und Blendet den gemappten Speicher aus.
// 	Ergibt 0 bei einem Fehler bzw. 1 wenn alle in Ordung war
int CMapFile::Unmap()
{
int		bExit;
int		iSub;


	if(!hMapObj)return 0;

	iSub	= ((int)pMapMem)&(iPageSize-1);
	bExit   = UnmapViewOfFile(pMapMem-iSub);					// Mapping aufheben

	pMapMem = 0;												// Zeiger auf Speicher
	iMapLen = 0;												// Lnge des Speichers


return bExit;
}


