//*****************************************************************************
//*
//*
//*		StringChain.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Erzeugt eine Kette von Strings die zustzlich mit Daten erweitert
//	werden kann. Die Datenblcke mssen immer die gleiche Lnge aufweisen
//	die Strings knnen unterschiedlich lang sein.
//	Zustzlich wird die Stringlnge gespeichert.
//	Alle Strings samt Daten werden dynamisch verwaltet, und in eigenen
//	Speicherbereichen zwischen gespeichert.
//
//	Funktionen:		char *Get				: Zeiger auf Text holen
//					void *Data				: Zeiger auf Datenbereich holen
//					void *DataGet			: Daten  holen
//					void *DataSet   		: Daten  setzen
//					bool  DataLength		: Verndert  Datalnge
//					int   Len				: Textlnge holen
//					int   Add				: Text und Daten hinzufgen (auch Ketten)
//					int   Del				: Einzelnen Text oder alles lschen
//					int   Count				: Anzahl der Texte in der Klasse
//					int   Find				: Text oder Wert in der Klasse suchen
//					int   Move				: Text in der Klasse verschieben
//					int   Chain     		: Erzeugt Textkette
//					void  Sort				: Eintrge sortieren
//					void  Upper				: Zu  Grobuchstaben umwandeln
//					void  Lower				: Zu Kleinbuchstaben umwandeln
//					int   Memory    		: Erzeugt Textseite mit /n/r
//					int   GetUserData 		: Schreibt ID-Nummer
//					int   GetUserData		: Liest    ID-Nummer
//					void *UserPtr(void*)	: Schreibt User-Zeiger
//					void *UserPtr()			: Liest    User-Zeiger
//					void  Change			: Tauscht zwei Eintrge
//					void  Set				: Text an ndern
//					int	  Plus				: Text an Text anhngen
//					void  TrimLeft 			: Leerzeichen links  entfernen
//					void  TrimRight			: Leerzeichen rechts entfernen
//
//	CONTAINER
//
//  int	CAdd(          char  *pStr  ,void *pData=0);	// String in Container  einfgen
//	int	CAdd(int  iLen,char  *pStr  ,void *pData=0);	// String in Container  einfgen
//	int	CAdd(		   char **pChain,void *pData=0);	// ByteKetten in Container  einfgen
//	int	CAdd(char cKey,char **pChain,void *pData=0);	// ByteKetten in Container  einfgen
//
//	int	ContainerInit (int iDepth=16,int iMode=0);		// Container initialisieren
//	int	ContainerCheck(char *pStr);						// Container prfen
//	int ContainerCut  ();								// Container krzen
//
//	STACK
//
//	int	  Push(		    char *pStr  ,void *pData=0);	// String auf Stapel legen
//	int	  Push(int iLen,char *pStr  ,void *pData=0);	// String auf Stapel legen
//
//	FIFO
//
//	int	  FifoIn (int iLen,char  *pStr,void *pData=0);	// String in  Fifo legen
//	int	  FifoOut(	       char **pStr,void *pData=0);	// String aus Fifo holen
//
//	Das Modul "MemFunctions.cpp" wird mit verwendet !
//
//
#ifdef		GLOBAL_CONFIG
#define 	STRINGCHAIN_CPP
#include	<GlobalConfig.h>
#undef		STRINGCHAIN_CPP
#endif

#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	"StringChain.h"

#if        _MSC_VER >= 1000
#pragma		warning( disable :4302 )
#pragma		warning( disable :4311 )
#pragma		warning( disable :4312 )
#endif

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		Z
#undef 		Z
#endif

#ifdef		i64
#undef 		i64
#endif

#ifdef		u64
#undef 		u64
#endif

#ifdef		u16
#undef 		u16
#endif

#ifdef		u8
#undef 		u8
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		strzlen
#undef 		strzlen
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	strzlen		(int)wcslen
#define 	strzcmp		wcscmp
#define 	strzupr		wcsupr
#define 	strzlwr		wcslwr
#define 	pprintfz	pprintfvw
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	strzlen		(int)strlen
#define 	strzcmp		strcmp
#define 	strzupr		strupr
#define 	strzlwr		strlwr
#define 	pprintfz	pprintfva
#define 	memncmp		memcmp
#endif

#ifdef		linux
#define		i64	 		long long
#define		u64	 		unsigned long long
#define		u16	 		unsigned short
#define		u8	 		unsigned char
#else
#define		i64	 		__int64
#define		u64	 		unsigned __int64
#define		u16	 		unsigned short
#define		u8	 		unsigned char
#endif



//*****************************************************************************
//*
//*		Externe Funktionen
//*
//*****************************************************************************

#ifdef		CSC_NO_PPRINTF

#undef		pprintfz

#if		   !defined(linux) && defined(UNICODE)
#define		pprintfz(d,p,f,a)		_vsnwprintf((d)->pBuffer,(d)->iSize,f,a)
#elif	   !defined(linux)
#define		pprintfz(d,p,f,a)		_vsnprintf((d)->pBuffer,(d)->iSize,f,a)
#elif		defined(UNICODE)
#define 	pprintfz(d,p,f,a)		vswprintf((d)->pBuffer,(d)->iSize,f,a)
#else
#define 	pprintfz(d,p,f,a)		vsnprintf((d)->pBuffer,(d)->iSize,f,a)
#endif

#else

typedef	int (_cdecl *pprintf_proca)(void *pParam,const char    *pBuffer,int iSize);
typedef	int (_cdecl *pprintf_procw)(void *pParam,const wchar_t *pBuffer,int iSize);

extern "C" int  _cdecl pprintfva(void *pParam,pprintf_proca pProc,const char    *pFormat,va_list pArgs);
extern "C" int  _cdecl pprintfvw(void *pParam,pprintf_procw pProc,const wchar_t *pFormat,va_list pArgs);

#endif



#ifdef		CSC_NO_MEMGCMP

#if		   !defined(linux) && defined(UNICODE)
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2           ){return wcsicmp    (pBuffer1,pBuffer2       );}
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2,int iCount){return wcsnicmp   (pBuffer1,pBuffer2,iCount);}
#elif	   !defined(linux)
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2           ){return stricmp    (pBuffer1,pBuffer2       );}
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2,int iCount){return strnicmp   (pBuffer1,pBuffer2,iCount);}
#elif		defined(UNICODE)
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2           ){return wcscasecmp (pBuffer1,pBuffer2       );}
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2,int iCount){return wcsncasecmp(pBuffer1,pBuffer2,iCount);}
#else
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2           ){return strcasecmp (pBuffer1,pBuffer2       );}
inline int  memgcmp(const zchar *pBuffer1,const zchar *pBuffer2,int iCount){return strncasecmp(pBuffer1,pBuffer2,iCount);}
#endif

#else

int  _cdecl memgcmp(const char    *pBuffer1,const char     *pBuffer2);
int  _cdecl memgcmp(const wchar_t *pBuffer1,const wchar_t  *pBuffer2);
int  _cdecl memgcmp(const char    *pBuffer1,const char     *pBuffer2,int iCount);
int  _cdecl memgcmp(const wchar_t *pBuffer1,const wchar_t  *pBuffer2,int iCount);

#endif


typedef struct
	{
	zchar  *pBuffer;
	int		iSize;
	int		iPos;
	int		iNew;
	}PrintfData;


//*****************************************************************************
//*
//*		memncmp
//*
//*****************************************************************************
//	Vergleichsfunktion fr Unicode-Text
#ifdef UNICODE
static int memncmp(const zchar *pText1,const zchar *pText2,int iBytes)
{
int			iDiff;
int			iPos;


	iBytes /= sizeof(zchar);

	for(iPos=0;iPos<iBytes;iPos++)
		{
			iDiff = pText1[iPos]-pText2[iPos];
		if(!iDiff)continue;

		return iDiff;
		}


return 0;
}
#endif

//******************************************************************************
//*
//*		strlwr
//*
//******************************************************************************
//	Text in Kleinbuchstaben konvertieren (nur fr Nichr Microsoft-Compiler)
#ifndef _MSC_VER

static inline void strlwr(char *pText)
{

	while(*pText)
		{
		if(*pText<='Z' && *pText>='A')*pText+=0x20;
		}

}

static inline void wcslwr(wchar_t *pText)
{

	while(*pText)
		{
		if(*pText<='Z' && *pText>='A')*pText+=0x20;
		}

}

#endif

//******************************************************************************
//*
//*		strupr
//*
//******************************************************************************
//	Text in Grobuchstaben konvertieren (nur fr Nichr Microsoft-Compiler)
#ifndef _MSC_VER

static inline void strupr(char *pText)
{

	while(*pText)
		{
		if(*pText>='a' && *pText<='z')*pText-=0x20;
		}

}

static inline void wcsupr(wchar_t *pText)
{

	while(*pText)
		{
		if(*pText>='a' && *pText<='z')*pText-=0x20;
		}

}

#endif


//****************************************************************************
//*
//*		Makros
//*
//****************************************************************************
#define uint	unsigned

#define	FieldAdd(iCnt)															\
	{																			\
	if(m_iCount+(iCnt)>=m_iMaxCount)											\
		{																		\
		char **pOld,**pTmp;														\
		m_iMaxCount=((m_iMaxCount+(iCnt))*3+32)>>1;								\
		pOld=m_pField;															\
		pTmp=new char*[m_iMaxCount];											\
		memcpy(pTmp,pOld,m_iCount*sizeof(char*));								\
		m_pField=pTmp;															\
		if(pOld)delete pOld;													\
		}																		\
	}

#define	FieldMove(iNext,iFrom,iCnt)												\
	{																			\
	int iTmp=iCnt;																\
	if(iTmp>0)memmove(m_pField+(iNext),m_pField+(iFrom),iTmp*sizeof(char*));	\
	}

#define	DelBlock(pPtr)															\
	{																			\
	pPtr -=	sizeof(int);														\
	delete pPtr;																\
	}

#define	NewBlock(pPtr,pStr,pData,iLen)											\
	{																			\
	int iStrLen=iLen;															\
	pPtr = new char[m_iDataSize+sizeof(int)+(iStrLen+1)*sizeof(zchar)];			\
	*(int*)pPtr=iStrLen;														\
	pPtr += sizeof(int);														\
																				\
	memcpy    (pPtr+m_iDataSize ,pStr,iStrLen *sizeof(zchar));					\
	*(zchar*) (pPtr+m_iDataSize+      iStrLen *sizeof(zchar))=0;				\
																				\
	if(m_iDataSize)																\
		{																		\
		if(pData)memcpy(pPtr,pData,m_iDataSize);								\
		else	 memset(pPtr,0    ,m_iDataSize);								\
		}																		\
	}

#define	NewBlock1(pPtr,pData,iLen)												\
	{																			\
	int iStrLen=iLen;															\
	pPtr = new char[m_iDataSize+sizeof(int)+(iStrLen+1)*sizeof(zchar)];			\
	*(int*)pPtr=iStrLen;														\
	pPtr += sizeof(int);														\
																				\
	*(zchar*) (pPtr+m_iDataSize+(iStrLen)*sizeof(zchar))=0;						\
																				\
	if(m_iDataSize)																\
		{																		\
		if(pData)memcpy(pPtr,pData,m_iDataSize);								\
		else	 memset(pPtr,0    ,m_iDataSize);								\
		}																		\
	}

#define	NewBlock2(pPtr,pStr,pData,iLen,iSize)									\
	{																			\
	int iStrLen=iLen;															\
	pPtr = new char[iSize+sizeof(int)+(iStrLen+1)*sizeof(zchar)];				\
	*(int*)pPtr=iStrLen;														\
	pPtr += sizeof(int);														\
																				\
	memcpy    (pPtr+iSize ,pStr,iStrLen *sizeof(zchar));						\
	*(zchar*) (pPtr+iSize+      iStrLen *sizeof(zchar))=0;						\
																				\
	if(iSize)																	\
		{																		\
		if(pData)memcpy(pPtr,pData,iSize);										\
		else	 memset(pPtr,0    ,iSize);										\
		}																		\
	}



//*****************************************************************************
//*
//*		StringChainPrintf
//*
//*****************************************************************************
//	Ausgabefunktion fr alle Formatausgaben.
static int StringChainPrintf(void *pParam,const zchar *pText,int iSize)
{
PrintfData *pData=(PrintfData*)pParam;
zchar	   *pNew;
int			iLen;


	if(iSize<=0)return 0;


	iLen = pData->iPos+iSize;

	if(iLen>pData->iSize)
		{
		iLen += pData->iSize/2+256;
		pNew  = new zchar[iLen];
		memcpy(pNew,pData->pBuffer,pData->iPos*sizeof(zchar));
		if(pData->iNew)delete pData->pBuffer;
		pData->iSize = iLen;
		pData->iNew  = 1;
		}

	memcpy(pData->pBuffer+pData->iPos,pText,iSize*sizeof(zchar));
	pData->iPos += iSize;


return iSize;
}

//****************************************************************************
//*
//*		Konstruktor
//*
//****************************************************************************
//	iNewDataLen		: Ist die Lnge des Datenbereiches
CStringChain::CStringChain(int iNewDataLen)
{

	m_pField	= 0;									// Zeigerfeld
	m_iCount	= 0;									// Anzahl der Texte
	m_iMaxCount	= 0;									// Feldgre
	m_iDataSize	= iNewDataLen;							// Lnge fr Zusatzdaten

}

//****************************************************************************
//*
//*		Destruktor
//*
//****************************************************************************
CStringChain::~CStringChain()
{

	Del();  											// Texte lschen

}

//****************************************************************************
//*
//*		Add
//*
//****************************************************************************
//	Fgt einen Eintrag in die Klasse ein, der zuvor mit Peek(...) geholt wurde.
//	pEntry		: Ist der Zeiger auf den Eintrag.
//	iPos		: Ist die Einfgeposition fr den Text. Wenn diese Position
//				  kleiner Null ist, wird der Eintrag am Ende angehngt.
//	Ergibt die Position an der der Text eingefgt wurde.
int	CStringChain::Add(void *pEntry,int iPos)
{

	if(!pEntry)return -1;
	if((uint)iPos>(uint)m_iCount)iPos=m_iCount;			// Position ok ?

	FieldAdd(1);										// Ist Zeiger-Feld zuklein ?
	FieldMove(iPos+1,iPos,m_iCount-iPos);				// Feld verschieben
	m_pField[iPos]=(char*)pEntry;
	m_iCount++;


return iPos;
}

//****************************************************************************
//*
//*		Add
//*
//****************************************************************************
//	Fgt einen Text in die Klasse ein.
//	pStr		: Ist der Zeiger auf den Text. Wenn dieser Zeiger NULL ist,
//				  dann wird "" eingefgt.
//	iPos		: Ist die Einfgeposition fr den Text. Wenn diese Position
//				  kleiner Null ist, wird der Text am Ende angehngt.
//	pData		: Ist der Zeiger auf den Datenbereich zu dem Text. Wenn dieser
//				  Zeiger NULL ist dann wird der Datenbereich auf Null gesetzt.
//	Ergibt die Position an der der Text eingefgt wurde.
int	CStringChain::Add(const zchar *pStr,int iPos,const void *pData)
{
char    *pPtr;
int		 iLen;



	if(!pStr){iLen=0;pStr=Z("");}						// Ist es ein NULL-String
	else	 iLen=(int)strzlen(pStr);
	if((uint)iPos>(uint)m_iCount)iPos=m_iCount;			// Position ok ?

	NewBlock(pPtr,pStr,pData,iLen);						// Neuen Block erzeugen

	FieldAdd(1);										// Ist Zeiger-Feld zuklein ?
	FieldMove(iPos+1,iPos,m_iCount-iPos);				// Feld verschieben
	m_pField[iPos]=pPtr;
	m_iCount++;


return iPos;
}

//****************************************************************************
//*
//*		Add
//*
//****************************************************************************
//	Fgt einen Text in die Klasse ein mit einer Lngenangabe.
//	iLen		: Ist die Lnge des Textes in Zeichen.
//	pStr		: Ist der Zeiger auf den Text. Wenn dieser Zeiger NULL ist,
//				  dann wird "" eingefgt.
//	iPos		: Ist die Einfgeposition fr den Text. Wenn diese Position
//				  kleiner Null ist, wird der Text am Ende angehngt.
//	pData		: Ist der Zeiger auf den Datenbereich zu dem Text. Wenn dieser
//				  Zeiger NULL ist dann wird der Datenbereich auf Null gesetzt.
//	Ergibt die Position an der der Text eingefgt wurde.
int	CStringChain::Add(int iLen,const zchar *pStr,int iPos,const void *pData)
{
char   *pPtr;


	if(!pStr){pStr=Z("");iLen=0;}						// Ist es ein NULL-String
	if((uint)iPos>(uint)m_iCount)iPos=m_iCount;			// Position ok ?

	NewBlock(pPtr,pStr,pData,iLen);						// Neuen Block erzeugen

	FieldAdd(1);										// Ist Zeiger-Feld zuklein ?
	FieldMove(iPos+1,iPos,m_iCount-iPos);				// Feld verschieben
	m_pField[iPos]=pPtr;
	m_iCount++;


return iPos;
}

//****************************************************************************
//*
//*		Add
//*
//****************************************************************************
//	Fgt eine Textkette in die Klasse ein
//	pChain		: Ist der Zeiger auf die Textkette.
//	iPos		: Ist die Einfgeposition fr die Textkette. Wenn diese Position
//				  kleiner Null ist, wird der Text am Ende angehngt.
//	iStart		: Ist der erste Text aus der Kette der eingefgt werden soll.
//	iStop		: Ist der letzte Text aus der Kette der eingefgt werden soll.
//				  Wenn dieser Wert kleiner Null ist, wird alles bis zum Ende
//				  der Kette iengefgt.
//	Ergibt die Position ab der die Textkette eingefgt wurde.
int	CStringChain::Add(CStringChain *pChain,int iPos,int iStart,int iStop)
{
char   *pMem;
int		iCnt;
int		iNum;



	if(!pChain)return -1;								// Ist die Kette leer

	   iCnt=pChain->Count();
	if(iCnt<=0)return -1;

	if(iStart<  0)iStart=0;
	if((uint)iStop>(uint)iCnt)iStop=iCnt;
	if(iStop <=iStart)return -1;

	if((uint)iPos>(uint)m_iCount)iPos=m_iCount;			// Position ok ?

	iNum = iStop-iStart;

	FieldAdd(iNum);										// Ist Zeiger-Feld zuklein ?
	FieldMove(iPos+iNum,iPos,m_iCount-iPos);			// Feld verschieben
	m_iCount+=iNum;

	for(;iStart<iStop;iStart++,iPos++)					// Neue Blcke erzeugen
		{
		NewBlock(pMem,pChain->GetNone(iStart),pChain->DataNone(iStart),pChain->LenNone(iStart));
		m_pField[iPos]=pMem;
		}


return iPos-iNum;
}

//****************************************************************************
//*
//*		KAdd
//*
//****************************************************************************
//	Fgt eine Textkette bei Position ein.
//	cKey	: Ist das Trennzeichen fr die Texte in der Textkette.
//	pChain	: Ist der Zeiger auf die Textkette
//	iPos	: Ist die Einfgeposition. Wenn iPos -1 ist dann wird der Text
//			  am Ende angehngt.
//	pData	: Ist der Zeiger auf die Datenbereiche die mit eingefgt werden.
//	Ergibt die Position im der pChain eingefgt wird bzw. -1 wenn pChain leer war.
int	CStringChain::KAdd(zchar cKey,const zchar *pChain,int iPos,const void *pData)
{
int		i,iLen;



	if(!pChain || !*pChain)return -1;					// Ist String ok ?

	if((uint)iPos>(uint)m_iCount)iPos=m_iCount;			// Position ok ?

	for(i=iPos;*pChain;i++)								// Kettentexte einfgen
		{
		for(iLen=0;pChain[iLen] && pChain[iLen]!=cKey;iLen++);
		Add(iLen,pChain,i,pData);

		pChain += iLen;

		if(!cKey || *pChain)
			{
			pChain++;
			}
		}


return iPos;
}

//****************************************************************************
//*
//*		KAdd
//*
//****************************************************************************
//	Fgt eine Textkette bei Position ein.
//	cKey	: Ist das Trennzeichen fr die Texte in der Textkette.
//	iMax	: Ist die Lnge der Textkette in Zeichen
//	pChain	: Ist der Zeiger auf die Textkette
//	iPos	: Ist die Einfgeposition. Wenn iPos -1 ist dann wird der Text
//			  am Ende angehngt.
//	pData	: Ist der Zeiger auf die Datenbereiche die mit eingefgt werden.
//	Ergibt die Position im der pChain eingefgt wird bzw. -1 wenn pChain leer war.
int	CStringChain::KAdd(zchar cKey,int iMax,const zchar *pChain,int iPos,const void *pData)
{
int		i,iLen;



	if(!pChain || iMax<0)return -1;						// Ist String ok ?

	if((uint)iPos>(uint)m_iCount)iPos=m_iCount;			// Position ok ?

	for(i=iPos;iMax>0;i++)								// Kettentexte einfgen
		{
		for(iLen=0;pChain[iLen] && pChain[iLen]!=cKey && iLen<iMax;iLen++);
		Add(iLen,pChain,i,pData);

		pChain += iLen;
		iMax   -= iLen;

		if(!cKey || *pChain)
			{
			pChain++;
			iMax--;
			}
		}


return iPos;
}

//****************************************************************************
//*
//*		Set
//*
//****************************************************************************
//	ndert einen Text bei einer bestimmten Position in der Textkette.
//	pStr		: Ist der Zeiger auf den neuen Text.
//	iPos		: Ist die Position der Textes der gendert werden soll.
//	pData		: Ist der Zeiger auf den Datenbereich zu dem Text. Wenn dieser
//				  Zeiger NULL ist dann wird der Datenbereich nicht verndert.
//	Ergibt die Position der genderten Texte ober -1 bei einem Fehler.
int	CStringChain::Set(const zchar *pStr,int iPos,const void *pData)
{
char   *pPtr;
char   *pOld;
int		iLen;


	if((uint)iPos>=(uint)m_iCount)return -1;			// Position ok ?
	if(!pStr){iLen=0;pStr=Z("");}						// Ist es ein NULL-String
	else	  iLen=(int)strzlen(pStr);



	pOld=m_pField[iPos];
	if(!pData)pData=pOld;

	NewBlock(pPtr,pStr,pData,iLen);						// Neuen Block erzeugen

	m_pField[iPos] = pPtr;

	DelBlock(pOld);


return iPos;
}

//****************************************************************************
//*
//*		Set
//*
//****************************************************************************
//	ndert einen Text bei einer bestimmten Position in der Textkette.
//	iLen		: Ist die Lnge des Textes in Zeichen.
//	pStr		: Ist der Zeiger auf den neuen Text.
//	iPos		: Ist die Position der Textes der gendert werden soll.
//	pData		: Ist der Zeiger auf den Datenbereich zu dem Text. Wenn dieser
//				  Zeiger NULL ist dann wird der Datenbereich nicht verndert.
//	Ergibt die Position der genderten Texte ober -1 bei einem Fehler.
int	CStringChain::Set(int iLen,const zchar *pStr,int iPos,const void *pData)
{
char   *pPtr,*pOld;


	if((uint)iPos>=(uint)m_iCount)return -1;			// Position ok ?
	if(!pStr){pStr=Z("");iLen=0;}						// Ist es ein NULL-String

	pOld=m_pField[iPos];
	if(!pData)pData=pOld;

	NewBlock(pPtr,pStr,pData,iLen);						// Neuen Block erzeugen

	m_pField[iPos] = pPtr;

	DelBlock(pOld);


return iPos;
}

//****************************************************************************
//*
//*		Change
//*
//****************************************************************************
//	Tauscht zwei Texte in der Kette gegeneinander aus.
//	iPos1		: Ist die Position des ersten  Textes
//	iPos2		: Ist die Position des zweiten Textes
void CStringChain::Change(int iPos1,int iPos2)
{
char   *pMem1,*pMem2;


	if(m_iCount<=0)return;
	if((uint)iPos1>=(uint)m_iCount)iPos1=m_iCount-1;	// Position ok ?
	if((uint)iPos2>=(uint)m_iCount)iPos2=m_iCount-1;	// Position ok ?

	pMem1=m_pField[iPos1];
	pMem2=m_pField[iPos2];
		  m_pField[iPos1]=pMem2;
		  m_pField[iPos2]=pMem1;

}

//****************************************************************************
//*
//*		Get
//*
//****************************************************************************
//	Einen Text von einer Position abfragen und in einen Puffer speichern.
//	Der Speicher fr den Puffer muss nacherher mit delete freigegeben werden.
//	iPos	: Ist die Position des Textes der abgefragt werden soll.
//	pStr	: Hier wird der Zeiger auf den Puffer gespeichert.
//			  Bei Fehlern wird NULL gespeichert.
//	Ergibt die Lnge des Strings bzw -1 bei einem Fehler.
int CStringChain::Get(int iPos,zchar **pStr)const
{
char  *pPtr;
int	   iLen;



	if((uint)iPos>=(uint)m_iCount)						// Auer Bereich ?
		{
		*pStr=0;
		return -1;
		}

	pPtr  = m_pField[iPos];
	iLen  = *(int*)(pPtr-sizeof(int));

		   *pStr = new zchar[iLen+1];
	memcpy(*pStr,pPtr+m_iDataSize,(iLen+1)*sizeof(zchar));


return iLen;
}

//****************************************************************************
//*
//*		Get
//*
//****************************************************************************
//	Einen Text von einer Position abfragen und in einen Puffer speichern.
//	Der Speicher fr den Puffer muss nacherher mit delete freigegeben werden.
//	iPos	: Ist die Position des Textes der abgefragt werden soll.
//	pStr	: Ist der Zeiger auf den Textpuffer.
//	iMax	: Ist die gre des Puffer in Zeichen.
//	Ergibt die Lnge des Strings bzw -1 bei einem Fehler.
int CStringChain::Get(int iPos,zchar *pStr,int iMax)const
{
char  *pPtr;
int	   iLen;



	if((uint)iPos>=(uint)m_iCount)						// Auer Bereich ?
		{
		if(iMax>0)pStr[0]=0;
		return -1;
		}

	pPtr  = m_pField[iPos];
	iLen  = 1+*(int*)(pPtr-sizeof(int));

	if(iLen>iMax)										// Ist Puffer zuklein ?
		{
		if(iMax>0)pStr[0]=0;
		return -1;
		}

	memcpy(pStr,pPtr+m_iDataSize,iLen*sizeof(zchar));	// Text kopieren


return iLen-1;
}

//****************************************************************************
//*
//*		DataGet
//*
//****************************************************************************
//	Kopiert den Datenbereich eines Textes in einen Puffer
//	iPos	: Ist die Position des Textes der abgefragt werden soll.
//	pData	: Ist der Zeiger auf den Datenpuffer.
//	Ergibt den Zeiger auf den Datenbereich des Textes oder NULL bei einem Fehler.
void *CStringChain::DataGet(int iPos,void *pData)const
{
void   *pPtr;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?

	pPtr = m_pField[iPos];
	memcpy(pData,pPtr,m_iDataSize);


return pPtr;
}


//****************************************************************************
//*
//*		DataSet
//*
//****************************************************************************
//	ndert den Datenbereich eines Textes.
//	iPos	: Ist die Position des Textes der abgefragt werden soll.
//	pData	: Ist der Zeiger auf den neuen Datenbereich.
//	Ergibt den Zeiger auf den Datenbereich des Textes oder NULL bei einem Fehler.
void *CStringChain::DataSet(int iPos,const void *pData)
{
void	*pPtr;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?

	pPtr = m_pField[iPos];
	memcpy(pPtr,pData,m_iDataSize);


return pPtr;
}


//****************************************************************************
//*
//*		DataSetPtr
//*
//****************************************************************************
//	ndert den ersten Zeiger des Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	pData	: Ist der neue Zeiger
//	Ergibt den alten Wert oder Null bei einem Fehler.
void *CStringChain::DataSetPtr(int iPos,const void *pData)
{
void	**pPtr;
void	 *pOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if( m_iDataSize<sizeof(void*))return 0;				// Zu kleiner Speicher


	pPtr=(void**)(m_pField[iPos]);
	pOld=*pPtr;
		 *pPtr=(void*)pData;


return pOld;
}

//****************************************************************************
//*
//*		DataSet
//*
//****************************************************************************
//	ndert den ersten INT-Wert des Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	iValue	: Ist der neue INT-Wert
//	Ergibt den alten Wert oder Null bei einem Fehler.
int CStringChain::DataSet(int iPos,int iValue)
{
int    *pPtr;
int		iOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if( m_iDataSize<  sizeof(int))return 0;				// Zu kleiner Speicher



	pPtr  = (int*)(m_pField[iPos]);
	iOld  = *pPtr;
	*pPtr = iValue;


return iOld;
}

//****************************************************************************
//*
//*		DataSetQ
//*
//****************************************************************************
//	ndert den ersten QWORD-Wert des Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	iValue	: Ist der neue QWORD-Wert
//	Ergibt den alten Wert oder Null bei einem Fehler.
i64 CStringChain::DataSetQ(int iPos,i64 iValue)
{
i64    *pPtr;
i64		iOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if( m_iDataSize<  sizeof(i64))return 0;				// Zu kleiner Speicher



	pPtr  = (i64*)(m_pField[iPos]);
	iOld  = *pPtr;
	*pPtr = iValue;


return iOld;
}

//****************************************************************************
//*
//*		DataSetW
//*
//****************************************************************************
//	ndert den ersten WORD-Wert des Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	uValue	: Ist der neue WORD-Wert
//	Ergibt den alten Wert oder Null bei einem Fehler.
u16 CStringChain::DataSetW(int iPos,u16 uValue)
{
u16    *pPtr;
u16		uOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if( m_iDataSize<  sizeof(u16))return 0;				// Zu kleiner Speicher



	pPtr  = (u16*)(m_pField[iPos]);
	uOld  = *pPtr;
	*pPtr = uValue;


return uOld;
}

//****************************************************************************
//*
//*		DataSetB
//*
//****************************************************************************
//	ndert den ersten BYTE-Wert des Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	uValue	: Ist der neue BYTE-Wert
//	Ergibt den alten Wert oder Null bei einem Fehler.
u8  CStringChain::DataSetB(int iPos,u8 uValue)
{
u8     *pPtr;
u8 		uOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if( m_iDataSize<   sizeof(u8))return 0;				// Zu kleiner Speicher



	pPtr  = (u8*)(m_pField[iPos]);
	uOld  = *pPtr;
	*pPtr = uValue;


return uOld;
}

//****************************************************************************
//*
//*		DataSet
//*
//****************************************************************************
//	ndert einen INT-Wert im Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	iValue	: Ist der neue INT-Wert
//	uIndex	: Ist der Index des gesuchten INT-Wertes
//	Ergibt den alten Wert oder Null bei einem Fehler.
int CStringChain::DataSet(int iPos,int iValue,unsigned uIndex)
{
int    *pPtr;
int		iOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if((uint)m_iDataSize<sizeof(int)+sizeof(int)*uIndex)// Zu kleiner Speicher
		{
		return 0;
		}


	pPtr  = (int*)(m_pField[iPos]);
	pPtr += uIndex;
	iOld  = *pPtr;
	*pPtr = iValue;


return iOld;
}

//****************************************************************************
//*
//*		DataSetQ
//*
//****************************************************************************
//	ndert einen QWORD-Wert im Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	iValue	: Ist der neue QWORD-Wert
//	uIndex	: Ist der Index des gesuchten QWORD-Wertes
//	Ergibt den alten Wert oder Null bei einem Fehler.
i64 CStringChain::DataSetQ(int iPos,i64 iValue,unsigned uIndex)
{
i64    *pPtr;
i64		iOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if((uint)m_iDataSize<sizeof(i64)+sizeof(i64)*uIndex)// Zu kleiner Speicher
		{
		return 0;
		}


	pPtr  = (i64*)(m_pField[iPos]);
	pPtr += uIndex;
	iOld  = *pPtr;
	*pPtr = iValue;


return iOld;
}

//****************************************************************************
//*
//*		DataSetW
//*
//****************************************************************************
//	ndert einen WORD-Wert im Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	uValue	: Ist der neue WORD-Wert
//	uIndex	: Ist der Index des gesuchten WORD-Wertes
//	Ergibt den alten Wert oder Null bei einem Fehler.
u16 CStringChain::DataSetW(int iPos,u16 uValue,unsigned uIndex)
{
u16    *pPtr;
u16		uOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if((uint)m_iDataSize<sizeof(u16)+sizeof(u16)*uIndex)// Zu kleiner Speicher
		{
		return 0;
		}


	pPtr  = (u16*)(m_pField[iPos]);
	pPtr += uIndex;
	uOld  = *pPtr;
	*pPtr = uValue;


return uOld;
}

//****************************************************************************
//*
//*		DataSetB
//*
//****************************************************************************
//	ndert einen BYTE-Wert im Datenbereiches eines Textes.
//	iPos	: Ist die Position des Textes.
//	uValue	: Ist der neue BYTE-Wert
//	uIndex	: Ist der Index des gesuchten BYTE-Wertes
//	Ergibt den alten Wert oder Null bei einem Fehler.
u8 CStringChain::DataSetB(int iPos,u8 uValue,unsigned uIndex)
{
u8     *pPtr;
u8		uOld;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auer Bereich ?
	if((uint)m_iDataSize<sizeof(u8)+sizeof(u8)*uIndex)	// Zu kleiner Speicher
		{
		return 0;
		}


	pPtr  = (u8*)(m_pField[iPos]);
	pPtr += uIndex;
	uOld  = *pPtr;
	*pPtr = uValue;


return uOld;
}

//****************************************************************************
//*
//*		Peek
//*
//****************************************************************************
//	Entfernt einen einzelnen Text aus der Textkette ohen den Eintrag zu lschen.
//	iPos	: Ist die Position des zu lschenden Textes
//	Ergibt einen Zeiger auf den Eintrag oder NULL bei einem Fehler.
void *CStringChain::Peek(int iPos)
{
char   *pStr;


	if((uint)iPos>=(uint)m_iCount)return 0;				// Auerhalb des Bereichs

	m_iCount--;
	pStr = m_pField[iPos];

	FieldMove(iPos,iPos+1,m_iCount-iPos);				// Feld verschieben

return pStr;
}

//****************************************************************************
//*
//*		Free
//*
//****************************************************************************
//	Lscht einen Eintrag der zuvor mit Peek(...) geholt wurde,
//	oder mit Alloc(...) erstellt wurde.
void CStringChain::Free(void *pEntry)
{
char   *pStr;


	if(!pEntry)return;

	pStr = (char*)pEntry;

	DelBlock(pStr);										// Eintrag lschen


}

//*****************************************************************************
//*
//*		Alloc
//*
//*****************************************************************************
//	Erstellt einen einzelnen Eintrag der mit Add(void*) eingefgt werden kann
//	Fgt einen Text in die Klasse ein mit einer Lngenangabe.
//	iLen		: Ist die Lnge des Textes in Zeichen.
//	pStr		: Ist der Zeiger auf den Text. Wenn dieser Zeiger NULL ist,
//				  dann wird "" eingefgt.
//	iSize		: Ist die Gre des Datenfeldes
//				  kleiner Null ist, wird der Text am Ende angehngt.
//	pData		: Ist der Zeiger auf den Datenbereich zu dem Text. Wenn dieser
//				  Zeiger NULL ist dann wird der Datenbereich auf Null gesetzt.
//	Ergibt einen Zeiger auf den neuen Eintrag
void *CStringChain::Alloc(int iLen,const zchar *pStr,int iSize,const void *pData)
{
char   *pPtr;


	if(!pStr){pStr=Z("");iLen=0;}						// Ist es ein NULL-String

	NewBlock2(pPtr,pStr,pData,iLen,iSize);				// Neuen Block erzeugen


return pPtr;
}

//****************************************************************************
//*
//*		Del
//*
//****************************************************************************
//	Lscht einen Bereich von Texte aus der Textkette
//	iForm	: Ist die Position des ersten zu lschenden Textes
//	iTo		: Ist die Position des letzten zu lschenden Textes
//			  Wenn dieser Wert kleiner Null ist wird alles bis zum Ende gelscht.
//	Ergibt die Anzahl der gelschten Texte
int CStringChain::Del(int iForm,int iTo)
{
char    *pStr;
int		 iPos;


	if(iForm<0        )iForm=0;							// Auerhalb des Bereichs ?
	if(iForm>=m_iCount)return 0;						//		*
	if((uint)iTo>(uint)m_iCount)iTo=m_iCount;			//		*
	if(iTo<=iForm     )return 0;						// Ende vor Start ?


	for(iPos=iForm;iPos<iTo;iPos++)
		{
		   pStr=m_pField[iPos];
		if(pStr)DelBlock(pStr);
		}

	m_iCount -= iTo-iForm;

	FieldMove(iForm,iTo,m_iCount-iForm);				// Feld verschieben


return iTo-iForm;
}

//****************************************************************************
//*
//*		Del
//*
//****************************************************************************
//	Lscht einen einzelnen Text aus der Textkette
//	iPos	: Ist die Position des zu lschenden Textes
//	Ergibt die Anzahl der gelschten Texte
int	CStringChain::Del(int iPos)
{
char   *pStr;



	if((uint)iPos>=(uint)m_iCount)return 0;				// Auerhalb des Bereichs

	m_iCount--;
	pStr = m_pField[iPos];

	FieldMove(iPos,iPos+1,m_iCount-iPos);				// Feld verschieben

	DelBlock(pStr);										// Eintrag lschen




return 1;
}

//****************************************************************************
//*
//*		Del
//*
//****************************************************************************
//	Lscht alle Texte aus der Textkette
//	Ergibt 1 wenn Texte gelscht wurden
int	CStringChain::Del()
{
char   *pStr;


	if(m_iCount>0)
		{
		do	{											// Eintrge lschen
			m_iCount--;
			pStr=m_pField[m_iCount];
			if(pStr)DelBlock(pStr);
			}while(m_iCount>0);
		}

	if(m_pField)										// Das Feld lschen
		{
		delete m_pField;
		m_iMaxCount = 0;
		m_pField    = 0;
		}



return 1;
}


//****************************************************************************
//*
//*		Find
//*
//****************************************************************************
//	Sucht einen Text in der Textkette
//	iVal	: Ist der Wert in den Daten der auch stimmen muss
//	iLen	: Ist die Lnge des gesuchten Textes in Zeichen.
//	pStr	: Ist der Zeiger auf den zu suchenden Textes.
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	iMode	: Wenn nicht Null wird zwischen Gro und Kleinbuchstaben nicht unterscheiden.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::Find(int iVal,int iLen,const zchar *pStr,int iStart,int iStop,int iMode)const
{
char          **pPos;
zchar		   *pTxt;
unsigned		uNum;
unsigned		uTst;
unsigned i64	uVal;
unsigned i64	uCmp;


	if(m_iDataSize<4)return -1;							// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						// 	*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// 	*
	if(iStop<=iStart)return -1;							// Stop vor Start ?
	if(!pStr){pStr=Z("");iLen=0;}

	pPos = m_pField+iStart;

	if(iMode==CSC_MASK_TEST && iLen*sizeof(zchar)>=8)	// Keine Gro/Kleinunterscheidung + Schnelltest	8B
		{
		uCmp  = *(unsigned i64*)(pStr);
		pStr += 8/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(int*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uVal  = *(unsigned i64*)(pTxt);
			if((uVal^uCmp)&0xDFDFDFDFDFDFDFDF)continue;
			pTxt += 8/sizeof(zchar);
			if(!memgcmp(pStr,pTxt,iLen-8/sizeof(zchar)))return iStart;
			}
		}												// Keine Gro/Kleinunterscheidung + Schnelltest 4B
	else if(iMode==CSC_MASK_TEST && iLen*sizeof(zchar)>=4)
		{
		uTst  = *(unsigned*)(pStr);
		pStr += 4/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(int*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uNum  = *(unsigned*)(pTxt);
			if((uNum^uTst)&0xDFDFDFDF)continue;
			pTxt += 4/sizeof(zchar);
			if(!memgcmp(pStr,pTxt,iLen-4/sizeof(zchar)))return iStart;
			}
		}
	else if(iMode)										// Keine Gro/Kleinunterscheidung
		{
		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(int*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			if(!memgcmp(pStr,pTxt,iLen))return iStart;
			}
		}
	else if(iLen*sizeof(zchar)>=8)						// Schnelltest 8 Byte
		{
		uCmp  = *(unsigned i64*)(pStr);
		pStr += 8/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(int*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uVal  = *(unsigned i64*)(pTxt);
			if(uVal!=uCmp)continue;
			pTxt += 8/sizeof(zchar);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)-8))return iStart;
			}
		}
	else if(iLen*sizeof(zchar)>=4)						// Schnelltest 4 Byte
		{
		uTst  = *(unsigned*)(pStr);
		pStr += 4/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(int*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uNum  = *(unsigned*)(pTxt);
			if(uNum!=uTst)continue;
			pTxt += 4/sizeof(zchar);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)-4))return iStart;
			}
		}
	else{
		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(int*)((*pPos)            ))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)))return iStart;
			}
		}



return -1;
}

//****************************************************************************
//*
//*		FindQ
//*
//****************************************************************************
//	Sucht einen Text in der Textkette
//	iVal	: Ist der Wert in den Daten der auch stimmen muss
//	iLen	: Ist die Lnge des gesuchten Textes in Zeichen.
//	pStr	: Ist der Zeiger auf den zu suchenden Textes.
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	iMode	: Wenn nicht Null wird zwischen Gro und Kleinbuchstaben nicht unterscheiden.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::FindQ(i64 iVal,int iLen,const zchar *pStr,int iStart,int iStop,int iMode)const
{
char          **pPos;
zchar		   *pTxt;
unsigned		uNum;
unsigned		uTst;
unsigned i64	uVal;
unsigned i64	uCmp;


	if(m_iDataSize<8)return -1;							// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return -1;							// Stop vor Start ?
	if(!pStr){pStr=Z("");iLen=0;}

	pPos = m_pField+iStart;

	if(iMode==CSC_MASK_TEST && iLen*sizeof(zchar)>=8)	// Keine Gro/Kleinunterscheidung + Schnelltest	8B
		{
		uCmp  = *(unsigned i64*)(pStr);
		pStr += 8/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uVal  = *(unsigned i64*)(pTxt);
			if((uVal^uCmp)&0xDFDFDFDFDFDFDFDF)continue;
			pTxt += 8/sizeof(zchar);
			if(!memgcmp(pStr,pTxt,iLen-8/sizeof(zchar)))return iStart;
			}
		}												// Keine Gro/Kleinunterscheidung + Schnelltest 4B
	else if(iMode==CSC_MASK_TEST && iLen*sizeof(zchar)>=4)
		{
		uTst  = *(unsigned*)(pStr);
		pStr += 4/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uNum  = *(unsigned*)(pTxt);
			if((uNum^uTst)&0xDFDFDFDF)continue;
			pTxt += 4/sizeof(zchar);
			if(!memgcmp(pStr,pTxt,iLen-4/sizeof(zchar)))return iStart;
			}
		}
	else if(iMode)										// Keine Gro/Kleinunterscheidung
		{
		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			if(!memgcmp(pStr,pTxt,iLen))return iStart;
			}
		}
	else if(iLen*sizeof(zchar)>=8)						// Schnelltest 8 Byte
		{
		uCmp  = *(unsigned i64*)(pStr);
		pStr += 8/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uVal  = *(unsigned i64*)(pTxt);
			if(uVal!=uCmp)continue;
			pTxt += 8/sizeof(zchar);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)-8))return iStart;
			}
		}
	else if(iLen*sizeof(zchar)>=4)						// Schnelltest 4 Byte
		{
		uTst  = *(unsigned*)(pStr);
		pStr += 4/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uNum  = *(unsigned*)(pTxt);
			if(uNum!=uTst)continue;
			pTxt += 4/sizeof(zchar);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)-4))return iStart;
			}
		}
	else{
		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			if(iVal!=*(i64*)((*pPos)            ))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)))return iStart;
			}
		}




return -1;
}

//****************************************************************************
//*
//*		Find
//*
//****************************************************************************
//	Sucht einen Text in der Textkette
//	iLen	: Ist die Lnge des gesuchten Textes in Zeichen.
//	pStr	: Ist der Zeiger auf den zu suchenden Textes.
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	iMode	: Wenn nicht Null wird zwischen Gro und Kleinbuchstaben nicht unterscheiden.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::Find(int iLen,const zchar *pStr,int iStart,int iStop,int iMode)const
{
char		  **pPos;
zchar		   *pTxt;
unsigned		uNum;
unsigned		uTst;
unsigned i64	uVal;
unsigned i64	uCmp;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return -1;							// Stop vor Start ?
	if(!pStr){pStr=Z("");iLen=0;}

	pPos = m_pField+iStart;

	if(iMode==CSC_MASK_TEST && iLen*sizeof(zchar)>=8)	// Keine Gro/Kleinunterscheidung + Schnelltest	8B
		{
		uCmp  = *(unsigned i64*)(pStr);
		pStr += 8/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uVal  = *(unsigned i64*)(pTxt);
			if((uVal^uCmp)&0xDFDFDFDFDFDFDFDF)continue;
			pTxt += 8/sizeof(zchar);
			if(!memgcmp(pStr,pTxt,iLen-8/sizeof(zchar)))return iStart;
			}
		}												// Keine Gro/Kleinunterscheidung + Schnelltest 4B
	else if(iMode==CSC_MASK_TEST && iLen*sizeof(zchar)>=4)
		{
		uTst  = *(unsigned*)(pStr);
		pStr += 4/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uNum  = *(unsigned*)(pTxt);
			if((uNum^uTst)&0xDFDFDFDF)continue;
			pTxt += 4/sizeof(zchar);
			if(!memgcmp(pStr,pTxt,iLen-4/sizeof(zchar)))return iStart;
			}
		}
	else if(iMode)										// Keine Gro/Kleinunterscheidung
		{
		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			if(!memgcmp(pStr,pTxt,iLen))return iStart;
			}
		}
	else if(iLen*sizeof(zchar)>=8)						// Schnelltest 8 Byte
		{
		uCmp  = *(unsigned i64*)(pStr);
		pStr += 8/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uVal  = *(unsigned i64*)(pTxt);
			if(uVal!=uCmp)continue;
			pTxt += 8/sizeof(zchar);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)-8))return iStart;
			}
		}
	else if(iLen*sizeof(zchar)>=4)						// Schnelltest 4 Byte
		{
		uTst  = *(unsigned*)(pStr);
		pStr += 4/sizeof(zchar);

		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			uNum  = *(unsigned*)(pTxt);
			if(uNum!=uTst)continue;
			pTxt += 4/sizeof(zchar);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)-4))return iStart;
			}
		}
	else{
		for(;iStart<iStop;iStart++,pPos++)
			{											// Ungleiche Textlngen ?
			if(iLen!=*(int*)((*pPos)-sizeof(int)))continue;
			pTxt  = (zchar*)((*pPos)+m_iDataSize);
			if(!memncmp(pStr,pTxt,iLen*sizeof(zchar)))return iStart;
			}
		}


return -1;
}

//****************************************************************************
//*
//*		Find
//*
//****************************************************************************
//	Sucht einen INT-Wert in den Datenbereichen der Textkette
//	iValue	: Ist der gesuchte INT-Wert
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::Find(int iValue,int iStart,int iStop)const
{
char   **pPos;


	if(m_iDataSize<4)return -1;							// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return -1;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		if(*(int*)(*pPos)==iValue)return iStart;
		}


return -1;
}

//****************************************************************************
//*
//*		FindQ
//*
//****************************************************************************
//	Sucht einen INT-Wert in den Datenbereichen der Textkette
//	iValue	: Ist der gesuchte INT-Wert
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::FindQ(i64 iValue,int iStart,int iStop)const
{
char   **pPos;


	if(m_iDataSize<8)return -1;							// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return -1;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		if(*(i64*)(*pPos)==iValue)return iStart;
		}


return -1;
}

//****************************************************************************
//*
//*		Find
//*
//****************************************************************************
//	Sucht einen INT-Wert in den Datenbereichen der Textkette
//	iValue	: Ist der gesuchte INT-Wert
//	iOffset	: Ist der Offset in den Datenbereichen wo gesucht wird
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::Find(int iValue,int iOffset,int iStart,int iStop)const
{
char   **pPos;


	if(m_iDataSize<4+iOffset)return -1;					// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return -1;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		if(*(int*)((*pPos)+iOffset)==iValue)return iStart;
		}


return -1;
}

//****************************************************************************
//*
//*		FindQ
//*
//****************************************************************************
//	Sucht einen INT-Wert in den Datenbereichen der Textkette
//	iValue	: Ist der gesuchte INT-Wert
//	iOffset	: Ist der Offset in den Datenbereichen wo gesucht wird
//	iStart	: Ist die Startposition ab der gesucht wird.
//	iStop	: Ist die Endposition der Suche, bei -1 wird bis zum Ende gesucht.
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn Nichts gefunden wurde.
int	CStringChain::FindQ(i64 iValue,int iOffset,int iStart,int iStop)const
{
char   **pPos;


	if(m_iDataSize<8+iOffset)return -1;					// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return -1;						//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return -1;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		if(*(i64*)((*pPos)+iOffset)==iValue)return iStart;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht einen String mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iValue	: Ist der Wert nach dem zuerst gesucht wird (mit UINT Vergleich)
//	iLen	: Ist die Lnge des Strings
//	pStr	: Ist der Text nach dem gesucht wird
//	iStart	: Ist die Startposition
//	iStop	: Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 Gros und Kleinbuchstaben nicht unterscheiden
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//			  Bit4 != 0 Lngere Namen zuerst
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(int iValue,int iLen,const zchar *pStr,int iStart,int iStop,int iMode)const
{
char   *pPtr;
int		iHalf;
int		iLong;
int		iCmp;
int		iNum;
int		iVal;




	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iLong = (iMode&0x10)? -1:1;

	if(!pStr){pStr=Z("");iLen=0;}
	iStop--;

	if(iMode&1)
	for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iVal  = *(int*)pPtr;

		if(iVal!=iValue)
			{
			iCmp = ((uint)iValue<(uint)iVal)? -1:1;
			}
		else{
			   iNum=*(int*)(pPtr-sizeof(int));
			if(iNum<iLen)
				{   iCmp=memgcmp(pStr,(zchar*)(pPtr+m_iDataSize),iNum);
				if(!iCmp)iCmp=iLong;			   		}
			else{   iCmp=memgcmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen);
				if(!iCmp && iLen!=iNum)iCmp=-iLong;		}
			}

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(!iCmp     )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06)) return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iVal  = *(int*)pPtr;

		if(iVal!=iValue)
			{
			iCmp = ((uint)iValue<(uint)iVal)? -1:1;
			}
		else{
			   iNum=*(int*)(pPtr-sizeof(int));
			if(iNum<iLen)
				{   iCmp=memncmp(pStr,(zchar*)(pPtr+m_iDataSize),iNum*sizeof(zchar));
				if(!iCmp)iCmp= iLong;			   	            	}
			else{   iCmp=memncmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen*sizeof(zchar));
				if(!iCmp && iLen!=iNum)iCmp=-iLong;			   		}
			}

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(!iCmp     )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06)) return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFastQ
//*
//****************************************************************************
//	Sucht einen String mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iValue	: Ist der Wert nach dem zuerst gesucht wird (mit ULONGLONG Vergleich)
//	iLen	: Ist die Lnge des Strings
//	pStr	: Ist der Text nach dem gesucht wird
//	iStart	: Ist die Startposition
//	iStop	: Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 Gros und Kleinbuchstaben nicht unterscheiden
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//			  Bit4 != 0 Lngere Namen zuerst
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFastQ(i64 iValue,int iLen,const zchar *pStr,int iStart,int iStop,int iMode)const
{
char	   *pPtr;
int			iHalf;
int			iLong;
int			iNum;
int 		iCmp;
i64			iVal;




	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iLong = (iMode&0x10)? -1:1;

	if(!pStr){pStr=Z("");iLen=0;}
	iStop--;

	if(iMode&1)
	for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iVal  = *(i64*)pPtr;

		if(iVal!=iValue)
			{
			iCmp = ((u64)iValue<(u64)iVal)? -1:1;
			}
		else{
			   iNum=*(int*)(pPtr-sizeof(int));
			if(iNum<iLen)
				{   iCmp=memgcmp(pStr,(zchar*)(pPtr+m_iDataSize),iNum);
				if(!iCmp)iCmp= iLong;			   		            }
			else{   iCmp=memgcmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen);
				if(!iCmp && iLen!=iNum)iCmp=-iLong;			   		}
			}

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(!iCmp     )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06)) return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iVal  = *(i64*)pPtr;

		if(iVal!=iValue)
			{
			iCmp = ((u64)iValue<(u64)iVal)? -1:1;
			}
		else{
			   iNum=*(int*)(pPtr-sizeof(int));
			if(iNum<iLen)
				{   iCmp=memncmp(pStr,(zchar*)(pPtr+m_iDataSize),iNum*sizeof(zchar));
				if(!iCmp)iCmp= iLong;			   		}
			else{   iCmp=memncmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen*sizeof(zchar));
				if(!iCmp && iLen!=iNum)iCmp=-iLong;		}
			}

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(!iCmp     )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06)) return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht einen String mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iLen	: Ist die Lnge des Strings
//	pStr	: Ist der Text nach dem gesucht wird
//	iStart	: Ist die Startposition
//	iStop	: Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 Gros und Kleinbuchstaben nicht unterscheiden
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//			  Bit4 != 0 Lngere Namen zuerst
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(int iLen,const zchar *pStr,int iStart,int iStop,int iMode)const
{
char    *pPtr;
int		 iLong;
int		 iHalf;
int		 iCmp;
int		 iNum;



	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iLong=(iMode&0x10)? -1:1;

	if(!pStr){pStr=Z("");iLen=0;}
	iStop--;

	if(iMode&1)	for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iNum=*(int*)(pPtr-sizeof(int));
		if(iNum<iLen)
			{   iCmp=memgcmp(pStr,(zchar*)(pPtr+m_iDataSize),iNum);
			if(!iCmp)iCmp= iLong;			   		}
		else{   iCmp=memgcmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen);
			if(!iCmp && iLen!=iNum)iCmp=-iLong;		}

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(!iCmp     )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06)) return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iNum=*(int*)(pPtr-sizeof(int));
		if(iNum<iLen)
			{   iCmp=memncmp(pStr,(zchar*)(pPtr+m_iDataSize),iNum*sizeof(zchar));
			if(!iCmp)iCmp= iLong;			   		}
		else{   iCmp=memncmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen*sizeof(zchar));
			if(!iCmp && iLen!=iNum)iCmp=-iLong;		}

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(!iCmp     )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06)) return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop  =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart =iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;
			 continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht eine Zahl mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iValue	: Ist die Zahl nach der gesucht wird
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(int iValue,int iStart,int iStop,int iMode)const
{
char    *pPtr;
int		 iHalf;
int		 iCmp;
int		 iVal;



	if(m_iDataSize<4)return -1;							// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	if(iMode&1) for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(int*)pPtr;
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((uint)iValue<(uint)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(int*)pPtr;
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((int)iValue<(int)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >=  0)? iStart+1:iStart;
					 	  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFastQ
//*
//****************************************************************************
//	Sucht eine Zahl mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iValue	: Ist die Zahl nach der gesucht wird
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFastQ(i64 iValue,int iStart,int iStop,int iMode)const
{
char   *pPtr;
int		iHalf;
int 	iCmp;
i64		iVal;



	if(m_iDataSize<4)return -1;							// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	if(iMode&1) for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(i64*)pPtr;
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((u64)iValue<(u64)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >= 0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(i64*)pPtr;
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((i64)iValue<(i64)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >= 0)? iStart+1:iStart;
					 	  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht eine Zahl mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iValue	: Ist die Zahl nach der gesucht wird
//	iOffset : Ist der Offset im Datenbereich
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(int iValue,int iOffset,int iStart,int iStop,int iMode)const
{
char    *pPtr;
int		 iHalf;
int		 iCmp;
int		 iVal;


	if(m_iDataSize<4+iOffset)return -1;					// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	if(iMode&1) for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(int*)(pPtr+iOffset);
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((uint)iValue<(uint)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >= 0)? iStart+1:iStart;
					  	  return -1;
			}

		if(!(iMode&0x06)) return iStart+ iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(int*)(pPtr+iOffset);
		if(iVal==iValue)iCmp=0;
		else		    iCmp=(( int)iValue<( int)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >= 0)? iStart+1:iStart;
					  	  break;
			}

		if(!(iMode&0x06)) return iStart+ iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFastQ
//*
//****************************************************************************
//	Sucht eine Zahl mit der Schnell-Such-Methode
//	Die Liste muss sortiert sein
//	iValue	: Ist die Zahl nach der gesucht wird
//	iOffset : Ist der Offset im Datenbereich
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFastQ(i64 iValue,int iOffset,int iStart,int iStop,int iMode)const
{
char    *pPtr;
int		iHalf;
int		iCmp;
i64		iVal;


	if(m_iDataSize<4+iOffset)return -1;					// Kein Int-Wert gespeichert
	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	if(iMode&1) for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(i64*)pPtr;
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((u64)iValue<(u64)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >= 0)? iStart+1:iStart;
					  	  return -1;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}
	else for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];

		   iVal=*(i64*)pPtr;
		if(iVal==iValue)iCmp=0;
		else		    iCmp=((i64)iValue<(i64)iVal)? -1:1;

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp >= 0)? iStart+1:iStart;
					  	  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht einen String mit der Schnell-Such-Methode und einer speziellen Funktion
//	Die Liste muss sortiert sein
//	pData	: Erster Parameter der an die Vergleichsfunktion bergeben wird.
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	pCmp	: Ist die Vergleichsfunktion. Mit ihr werden die Datenbereiche der
//			  Eintrge mit dem Parameter verglichen:
//				>0 Der Parameter ist grer  als der Eintrag
//				=0 Der Parameter ist gleich  mit dem Eintrag
//				<0 Der Parameter ist kleiner als der Eintrag
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(const void *pData,int iStart,int iStop,int iMode,int (__cdecl*pCmp)(const void*,const void*))const
{
char   *pPtr;
int		iHalf;
int		iCmp;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iCmp  = pCmp(pData,pPtr);

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp  >= 0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht einen String mit der Schnell-Such-Methode und einer speziellen Funktion
//	Die Liste muss sortiert sein
//	pParam	: Letzter Parameter der an die Vergleichsfunktion bergeben wird.
//	pData	: Erster Parameter der an die Vergleichsfunktion bergeben wird.
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	pCmp	: Ist die Vergleichsfunktion. Mit ihr werden die Datenbereiche der
//			  Eintrge mit dem Parameter verglichen:
//				>0 Der Parameter ist grer  als der Eintrag
//				=0 Der Parameter ist gleich  mit dem Eintrag
//				<0 Der Parameter ist kleiner als der Eintrag
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(void *pParam,const void *pData,int iStart,int iStop,int iMode,int (__cdecl*pCmp)(const void*,const void*,void*))const
{
char   *pPtr;
int		iHalf;
int		iCmp;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iCmp  = pCmp(pData,pPtr,pParam);

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp  >= 0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht einen String mit der Schnell-Such-Methode und einer speziellen Funktion
//	Die Liste muss sortiert sein
//	pStr	: Erster Text der an die Vergleichsfunktion bergeben wird.
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	pCmp	: Ist die Vergleichsfunktion. Mit ihr werden die Datenbereiche der
//			  Eintrge mit dem Parameter verglichen:
//				>0 Der Parameter ist grer  als der Eintrag
//				=0 Der Parameter ist gleich  mit dem Eintrag
//				<0 Der Parameter ist kleiner als der Eintrag
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(const zchar *pStr,int iStart,int iStop,int iMode,int (__cdecl*pCmp)(const zchar*,const zchar*))const
{
char   *pPtr;
int		iHalf;
int		iCmp;



	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?


	iStop--;

	for(;;)
		{
		iHalf = (iStop-iStart+1)>>1;
		pPtr  = m_pField[iStart+iHalf];
		iCmp  = pCmp(pStr,(zchar*)(pPtr+m_iDataSize));

		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp  >= 0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		FindFast
//*
//****************************************************************************
//	Sucht einen String+Lnge mit der Schnell-Such-Methode und einer speziellen Funktion
//	Die Liste muss sortiert sein
//	iLen	: Ist die Lnge des Textes in Zeichen
//	pStr	: Erster Text der an die Vergleichsfunktion bergeben wird.
//	iStart  : Ist die Startposition
//	iStop   : Ist das Ende der Suche bei -1 die String Anzahl
//	iMode	: Bit0 != 0 unsigned Vergleich
//			  Bit1 != 0 Erste  Position finden
//			  Bit2 != 0 Position hinter letzter suchen
//			  Bit3 != 0 Einfge Position suchen
//	pCmp	: Ist die Vergleichsfunktion. Mit ihr werden die Datenbereiche der
//			  Eintrge mit dem Parameter verglichen:
//				>0 Der Parameter ist grer  als der Eintrag
//				=0 Der Parameter ist gleich  mit dem Eintrag
//				<0 Der Parameter ist kleiner als der Eintrag
//	Ergibt die Position wo der Text gefunden wurde bzw. -1 wenn nicht gefunden.
int	CStringChain::FindFast(int iLen,const zchar *pStr,int iStart,int iStop,int iMode,int (__cdecl*pCmp)(const zchar*,const zchar*,int))const
{
char   *pPtr;
int		iSize;
int		iHalf;
int		iCmp;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)								// Sind keine Eintrge vorhanden
		{
		if(iMode&0x08)return m_iCount;
		return -1;
		}

	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// Ist das Ende hinter Anzahl
	if(iStop<=iStart)return -1;							// Stop vor Start ?
	if(iLen<0)return -1;								// Ist die Textlnge zu kurz

	iStop--;

	for(;;)
		{
		iHalf	= (iStop-iStart+1)>>1;
		pPtr	= m_pField[iStart+iHalf];
		iSize	= *(int*)(pPtr-sizeof(int));


		if(iSize>iLen)
			{
				iCmp = pCmp(pStr,(zchar*)(pPtr+m_iDataSize),iLen);
			if(!iCmp)iCmp=1;
			}
		else if(iSize<iLen)
			{
				iCmp = pCmp(pStr,(zchar*)(pPtr+m_iDataSize),iSize);
			if(!iCmp)iCmp=-1;
			}
		else{
				iCmp = pCmp(pStr,(zchar*)(pPtr+m_iDataSize),iSize);
			}


		if(iHalf && iCmp)								// Weiter suchen
			{
			if(iCmp<0){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;continue;}
					   iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;continue;
			}

		if(!iHalf)										// Ende erreicht
			{
			if(iCmp==0   )return (iMode&0x04)? iStart+1:iStart;
			if(iMode&0x08)return (iCmp  >= 0)? iStart+1:iStart;
						  break;
			}

		if(!(iMode&0x06))return iStart+iHalf;			// Gefunden
		if(  iMode&0x02){iStop =iStart+iHalf-1;if(iStart>iStop)iStop =iStart;}
		else		    {iStart=iStart+iHalf+1;if(iStart>iStop)iStart=iStop ;}
			 iMode|=0x08;

		continue;
		}


return -1;
}

//****************************************************************************
//*
//*		SortCmp
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortCmp(CStringChain *pChain,int iPos1,int iPos2)
{
return strzcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortCmpG
//*
//****************************************************************************
//	Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortCmpG(CStringChain *pChain,int iPos1,int iPos2)
{
return memgcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortInt
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortInt(CStringChain *pChain,int iPos1,int iPos2)
{
int iVal1,iVal2;

	iVal1 = *(int*)(pChain->m_pField[iPos1]);
	iVal2 = *(int*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return  0;
}

//****************************************************************************
//*
//*		SortIntC
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortIntC(CStringChain *pChain,int iPos1,int iPos2)
{
int iVal1,iVal2;

	iVal1 = *(int*)(pChain->m_pField[iPos1]);
	iVal2 = *(int*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return strzcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortIntCG
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortIntCG(CStringChain *pChain,int iPos1,int iPos2)
{
int iVal1,iVal2;

	iVal1 = *(int*)(pChain->m_pField[iPos1]);
	iVal2 = *(int*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return memgcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortI64
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortI64(CStringChain *pChain,int iPos1,int iPos2)
{
i64 iVal1,iVal2;

	iVal1 = *(i64*)(pChain->m_pField[iPos1]);
	iVal2 = *(i64*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return  0;
}

//****************************************************************************
//*
//*		SortI64C
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortI64C(CStringChain *pChain,int iPos1,int iPos2)
{
i64 iVal1,iVal2;

	iVal1 = *(i64*)(pChain->m_pField[iPos1]);
	iVal2 = *(i64*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return strzcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortI64CG
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortI64CG(CStringChain *pChain,int iPos1,int iPos2)
{
i64 iVal1,iVal2;

	iVal1 = *(i64*)(pChain->m_pField[iPos1]);
	iVal2 = *(i64*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return memgcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortUInt
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortUInt(CStringChain *pChain,int iPos1,int iPos2)
{
unsigned int iVal1,iVal2;

	iVal1 = *(unsigned int*)(pChain->m_pField[iPos1]);
	iVal2 = *(unsigned int*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return  0;
}

//****************************************************************************
//*
//*		SortUIntC
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortUIntC(CStringChain *pChain,int iPos1,int iPos2)
{
unsigned int iVal1,iVal2;

	iVal1 = *(unsigned int*)(pChain->m_pField[iPos1]);
	iVal2 = *(unsigned int*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return strzcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortUIntCG
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortUIntCG(CStringChain *pChain,int iPos1,int iPos2)
{
unsigned int iVal1,iVal2;

	iVal1 = *(unsigned int*)(pChain->m_pField[iPos1]);
	iVal2 = *(unsigned int*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return memgcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortUI64
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortUI64(CStringChain *pChain,int iPos1,int iPos2)
{
u64 iVal1,iVal2;

	iVal1 = *(u64*)(pChain->m_pField[iPos1]);
	iVal2 = *(u64*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return  0;
}

//****************************************************************************
//*
//*		SortUI64C
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortUI64C(CStringChain *pChain,int iPos1,int iPos2)
{
u64 iVal1,iVal2;

	iVal1 = *(u64*)(pChain->m_pField[iPos1]);
	iVal2 = *(u64*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return strzcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortUI64CG
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortUI64CG(CStringChain *pChain,int iPos1,int iPos2)
{
u64 iVal1,iVal2;

	iVal1 = *(u64*)(pChain->m_pField[iPos1]);
	iVal2 = *(u64*)(pChain->m_pField[iPos2]);

	if(iVal1<iVal2)return -1;
	if(iVal1>iVal2)return  1;

return memgcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortLen
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortLen(CStringChain *pChain,int iPos1,int iPos2)
{
unsigned int iLen1,iLen2;

	iLen1 = *(unsigned int*)(pChain->m_pField[iPos1]-sizeof(int));
	iLen2 = *(unsigned int*)(pChain->m_pField[iPos2]-sizeof(int));

	if(iLen1<iLen2)return -1;
	if(iLen1>iLen2)return  1;

return  0;
}

//****************************************************************************
//*
//*		SortLenC
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortLenC(CStringChain *pChain,int iPos1,int iPos2)
{
unsigned int iLen1,iLen2;

	iLen1 = *(unsigned int*)(pChain->m_pField[iPos1]-sizeof(int));
	iLen2 = *(unsigned int*)(pChain->m_pField[iPos2]-sizeof(int));

	if(iLen1<iLen2)return -1;
	if(iLen1>iLen2)return  1;

return strzcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		SortLenCG
//*
//****************************************************************************
//	Standard Vergleichsfunktion fr CStringChain::Sort
int __cdecl CStringChain::SortLenCG(CStringChain *pChain,int iPos1,int iPos2)
{
unsigned int iLen1,iLen2;

	iLen1 = *(unsigned int*)(pChain->m_pField[iPos1]-sizeof(int));
	iLen2 = *(unsigned int*)(pChain->m_pField[iPos2]-sizeof(int));

	if(iLen1<iLen2)return -1;
	if(iLen1>iLen2)return  1;

return memgcmp(pChain->GetNone(iPos1),pChain->GetNone(iPos2));
}

//****************************************************************************
//*
//*		Sort
//*
//****************************************************************************
//	Sortiert eine Textkette mit einer Sortierfunktion
//	iStart	: Ist die Startposition ab der sortiert wird.
//	iStop	: Ist die Endposition bis zu der sortiert wird.
//			  Bei -1 wird bis zum Ende sortiert.
//	pCmp	: Ist die Vergleichsfunktion:
//					CmpXXX(CStringChain *pChain,int iPos1 ,int iPos2)
//					   =0 wenn iPos1==iPos2
//				       >0 wenn iPos1> iPos2
//				       <0 wenn iPos1< iPos2
//			  pCmp == CSC_SORT_CMP		alphabetisch sortieren
//			  pCmp == CSC_SORT_CASE		alphabetisch sortieren ohne Gros/Klein
//			  pCmp == CSC_SORT_LEN		nach Stringlnge
//			  pCmp == CSC_SORT_LENC		nach Stringlnge dann alphabetisch
//			  pCmp == CSC_SORT_LENCU	nach Stringlnge dann alphabetisch ohne Gros/Klein
//			  pCmp == CSC_SORT_INT		nach int  Zahl
//			  pCmp == CSC_SORT_INTC		nach int  Zahl dann alphabetisch
//			  pCmp == CSC_SORT_INTCU	nach int  Zahl dann alphabetisch ohne Gros/Klein
//			  pCmp == CSC_SORT_UINT		nach uint Zahl
//			  pCmp == CSC_SORT_UINTC	nach uint Zahl dann alphabetisch
//			  pCmp == CSC_SORT_UINTCU	nach uint Zahl dann alphabetisch ohne Gros/Klein
//	iDir	: Ist die Sortierrichtung.
void CStringChain::Sort(int	iStart,int iStop,int (__cdecl *pCmp)(CStringChain*,int,int),int iDir)
{

	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							// 		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// 		*
	if(iStop <=iStart)return;							// Stop vor Start ?



//************************* Qsort-Alogarytmus *********************************
int		u,o,m,c;
int		aMemU[30],aMemO[30];
int		iLevel;
char   *pPos;


	iStop--;

	if(iStart>=iStop)return;

	iLevel = 0;


for(;;)
	{
    m = (iStart+iStop)>>1;								// Mitte berechnen
    u =  iStart;
    o =  iStop+1;

	pPos=m_pField[m];
		 m_pField[m]=m_pField[u];
		 m_pField[u]=pPos;

	for(;;)
		{
		do  {
            u++;
			if(u>iStop)break;
			c=pCmp(this,u,iStart);
			if(iDir)c=-c;
			}while(c<=0);

		do  {
            o--;
			if(o<=iStart)break;
			c=pCmp(this,o,iStart);
			if(iDir)c=-c;
			}while(c>=0);

		if(o<u)break;

		pPos=m_pField[u];
			 m_pField[u]=m_pField[o];
			 m_pField[o]=pPos;
		}

	pPos=m_pField[iStart];
		 m_pField[iStart]=m_pField[o];
		 m_pField[o]=pPos;

	if(o-1-iStart >= iStop-u)
		{
		if(iStart+1<o)
			{
            aMemU[iLevel] = iStart;
            aMemO[iLevel] = o - 1;
			iLevel++;
			}

		if(u<iStop)
			{
			iStart = u;
            continue;
			}
		}
	else{
		if(u<iStop)
			{
			aMemU[iLevel] = u;
			aMemO[iLevel] = iStop;
			iLevel++;
			}

		if(iStart+1<o)
			{
			iStop = o-1;
			continue;
			}
		}

    if(iLevel<=0)										// Noch Ebenen vorhanden
		{
		return;
		}

	iLevel--;											// Eine Ebene absteigen
    iStart = aMemU[iLevel];
    iStop  = aMemO[iLevel];
	}

}

//****************************************************************************
//*
//*		Sort
//*
//****************************************************************************
//	Sortiert eine Textkette mit einer Sortierfunktion
//	pCmp	: Ist die Vergleichsfunktion:
//					CmpXXX(void *pParam,const void *pPtr1,const void *pPtr2)
//					   =0 wenn pPtr1==pPtr2
//				       >0 wenn pPtr1> pPtr2
//				       <0 wenn pPtr1< pPtr2
//	pParam	: Ist der Userparameter fr die Vergleichsfunktion
//	iOffset	: Ist der Offset fr die Suche in einem Datenfeld
//	iStart	: Ist die Startposition ab der sortiert wird.
//	iStop	: Ist die Endposition bis zu der sortiert wird.
//			  Bei -1 wird bis zum Ende sortiert.
//	iDir	: Ist die Sortierrichtung.
void CStringChain::Sort(int (__cdecl*pCmp)(void*,const void*,const void*),void *pParam,int iOffset,int iStart,int iStop,int iDir)
{

	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							// 		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// 		*
	if(iStop <=iStart)return;							// Stop vor Start ?



//************************* Qsort-Alogarytmus *********************************
int		u,o,m,c;
int		aMemU[30],aMemO[30];
int		iLevel;
char   *pMem;
char   *pPos;


	iStop--;

	if(iStart>=iStop)return;

	iLevel	= 0;

	for(;;)
		{
		m	 = (iStart+iStop)>>1;						// Mitte berechnen
		u	 =  iStart;
		o    =  iStop+1;

		pPos=m_pField[m];
			 m_pField[m]=m_pField[u];
			 m_pField[u]=pPos;
		pMem=m_pField[iStart]+iOffset;

		for(;;)
			{
			do  {
				u++;
				if(u>iStop)break;
				c=pCmp(pParam,m_pField[u]+iOffset,pMem);
				if(iDir)c=-c;
				}while(c<=0);

			do  {
				o--;
				if(o<=iStart)break;
				c=pCmp(pParam,m_pField[o]+iOffset,pMem);
				if(iDir)c=-c;
				}while(c>=0);

			if(o<u)break;

			pPos=m_pField[u];
				 m_pField[u]=m_pField[o];
				 m_pField[o]=pPos;
			}

		pPos=m_pField[iStart];
			 m_pField[iStart]=m_pField[o];
			 m_pField[o]=pPos;

		if(o-1-iStart >= iStop-u)
			{
			if(iStart+1<o)
				{
				aMemU[iLevel] = iStart;
				aMemO[iLevel] = o - 1;
				iLevel++;
				}

			if(u<iStop)
				{
				iStart = u;
				continue;
				}
			}
		else{
			if(u<iStop)
				{
				aMemU[iLevel] = u;
				aMemO[iLevel] = iStop;
				iLevel++;
				}

			if(iStart+1<o)
				{
				iStop = o-1;
				continue;
				}
			}

		if(iLevel<=0)									// Noch Ebenen vorhanden
			{
			return;
			}

		iLevel--;										// Eine Ebene absteigen
		iStart = aMemU[iLevel];
		iStop  = aMemO[iLevel];
		}

}

//****************************************************************************
//*
//*		Sort
//*
//****************************************************************************
//	Sortiert eine Textkette mit einer Sortierfunktion
//	pCmp	: Ist die Vergleichsfunktion:
//					CmpXXX(const char *pPtr1,const void *pPtr2)
//					   =0 wenn pPtr1==pPtr2
//				       >0 wenn pPtr1> pPtr2
//				       <0 wenn pPtr1< pPtr2
//	pParam	: Ist der Userparameter fr die Vergleichsfunktion
//	iOffset	: Ist der Offset fr die Suche in einem Datenfeld
//	iStart	: Ist die Startposition ab der sortiert wird.
//	iStop	: Ist die Endposition bis zu der sortiert wird.
//			  Bei -1 wird bis zum Ende sortiert.
//	iDir	: Ist die Sortierrichtung.
void CStringChain::Sort(int (__cdecl*pCmp)(const zchar*,const zchar*),int iStart,int iStop,int iDir)
{

	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							// 		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		// 		*
	if(iStop <=iStart)return;							// Stop vor Start ?



//************************* Qsort-Alogarytmus *********************************
int		u,o,m,c;
int		aMemU[30],aMemO[30];
int		iLevel;
zchar  *pMem;
char   *pPos;


	iStop--;

	if(iStart>=iStop)return;

	iLevel	= 0;

	for(;;)
		{
		m	 = (iStart+iStop)>>1;						// Mitte berechnen
		u	 =  iStart;
		o    =  iStop+1;

		pPos=m_pField[m];
			 m_pField[m]=m_pField[u];
			 m_pField[u]=pPos;
		pMem=(zchar*)(m_pField[iStart]+m_iDataSize);

		for(;;)
			{
			do  {
				u++;
				if(u>iStop)break;
				c=pCmp((zchar*)(m_pField[u]+m_iDataSize),pMem);
				if(iDir)c=-c;
				}while(c<=0);

			do  {
				o--;
				if(o<=iStart)break;
				c=pCmp((zchar*)(m_pField[o]+m_iDataSize),pMem);
				if(iDir)c=-c;
				}while(c>=0);

			if(o<u)break;

			pPos=m_pField[u];
				 m_pField[u]=m_pField[o];
				 m_pField[o]=pPos;
			}

		pPos=m_pField[iStart];
			 m_pField[iStart]=m_pField[o];
			 m_pField[o]=pPos;

		if(o-1-iStart >= iStop-u)
			{
			if(iStart+1<o)
				{
				aMemU[iLevel] = iStart;
				aMemO[iLevel] = o - 1;
				iLevel++;
				}

			if(u<iStop)
				{
				iStart = u;
				continue;
				}
			}
		else{
			if(u<iStop)
				{
				aMemU[iLevel] = u;
				aMemO[iLevel] = iStop;
				iLevel++;
				}

			if(iStart+1<o)
				{
				iStop = o-1;
				continue;
				}
			}

		if(iLevel<=0)									// Noch Ebenen vorhanden
			{
			return;
			}

		iLevel--;										// Eine Ebene absteigen
		iStart = aMemU[iLevel];
		iStop  = aMemO[iLevel];
		}

}

//****************************************************************************
//*
//*		Lower
//*
//****************************************************************************
//	Wandelt die Zeichenfolgen der Textkette in Kleinbuchstaben um.
//	iStart	: Ist die Startposition ab der umgewandelt wird.
//	iStop	: Ist die Endposition bis zu der umgewandelt wird.
//			  Bei -1 wird bis zum Ende umgewandelt.
void CStringChain::Lower(int iStart,int iStop)
{
char  **pPos;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		strzlwr((zchar*)(*pPos)+m_iDataSize);
		}


}

//****************************************************************************
//*
//*		Upper
//*
//****************************************************************************
//	Wandelt die Zeichenfolgen der Textkette in Grobuchstaben um.
//	iStart	: Ist die Startposition ab der umgewandelt wird.
//	iStop	: Ist die Endposition bis zu der umgewandelt wird.
//			  Bei -1 wird bis zum Ende umgewandelt.
void CStringChain::Upper(int iStart,int iStop)
{
char  **pPos;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		strzupr((zchar*)(pPos[0]+m_iDataSize));
		}


}

//****************************************************************************
//*
//*		TrimLeft
//*
//****************************************************************************
//	Krzt die Leerzeichen links weg (Space Tab /n /r)
//	iStart	: Ist die Startposition ab der umgewandelt wird.
//	iStop	: Ist die Endposition bis zu der umgewandelt wird.
//			  Bei -1 wird bis zum Ende umgewandelt.
void CStringChain::TrimLeft(int iStart,int iStop)
{
char   **pPos;
char    *pPtr;
zchar   *pStr;
zchar    cChar;
int		 iLen;
int		 iOff;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		pPtr = *pPos;
		iLen = *(int *)(pPtr-sizeof(int));
		pStr = (zchar*)(pPtr+m_iDataSize);

		for(iOff=0;iOff<iLen;iOff++)
			{
			   cChar=pStr[iOff];
			if(cChar==' ' || cChar=='\t' || cChar=='\n' || cChar=='\r')continue;
			break;
			}

		if(iOff)
			{
			memmove(pStr,pStr+iOff,(iLen-iOff+1)*sizeof(zchar));
			*(int*)(pPtr-sizeof(int)) = iLen-iOff;
			}
		}


}

//****************************************************************************
//*
//*		TrimRight
//*
//****************************************************************************
//	Krzt die Leerzeichen rechts weg (Space Tab /n /r)
//	iStart	: Ist die Startposition ab der umgewandelt wird.
//	iStop	: Ist die Endposition bis zu der umgewandelt wird.
//			  Bei -1 wird bis zum Ende umgewandelt.
void CStringChain::TrimRight(int iStart,int iStop)
{
char   **pPos;
char    *pPtr;
zchar   *pStr;
zchar    cChar;
int		 iLen;
int		 iOff;


	if(iStart<0)iStart=0;								// Auerhalb des Bereichs ?
	if(iStart>=m_iCount)return;							//		*
	if((uint)iStop>(uint)m_iCount)iStop=m_iCount;		//		*
	if(iStop<=iStart)return;							// Stop vor Start ?

	pPos = m_pField+iStart;

	for(;iStart<iStop;iStart++,pPos++)
		{
		pPtr = *pPos;
		iLen = *(int *)(pPtr-sizeof(int));
		pStr = (zchar*)(pPtr+m_iDataSize);

		for(iOff=iLen;iOff>0;iOff--)
			{
			   cChar = pStr[iOff-1];
			if(cChar==' ' || cChar=='\t' || cChar=='\n' || cChar=='\r')continue;
			break;
			}

		if(iOff!=iLen)
			{
			pStr[iOff]=0;
			*(int*)(pPtr-sizeof(int)) = iLen;
			}
		}

}

//****************************************************************************
//*
//*		Move
//*
//****************************************************************************
//	Verschieb eine Text in der Kette auf eine bestimmte Position.
//	iPos	: Ist die Position des zu verschiebenden Textes.
//			  Wenn -1 ist dann wird der letzte Text genommen.
//	iNum	: Ist die Zeilposition fr den Text
//			  Wenn -1 dann wird der Text nach hinten verschoben.
//	Ergibt die neue Position des Textes bzw. -1 bei einem Fehler.
int CStringChain::Move(int iPos,int iNum)
{
char	*pStr;



	if(m_iCount<=0 )return 0;
	if(iPos  == -1 )iPos = m_iCount-1;
	if(iNum  == -1 )iNum = m_iCount-1;
	if(iNum  ==iPos)return iNum;

	if((uint)iNum>=(uint)m_iCount)return -1;
	if((uint)iPos>=(uint)m_iCount)return -1;


	pStr=m_pField[iPos];

	if(iNum>iPos)									//	Feld verschieben
		{
		FieldMove(iPos,iPos+1,iNum-iPos);
		}
	else{
		FieldMove(iNum+1,iNum,iPos-iNum);
		}

	m_pField[iNum] = pStr;


return iNum;
}

//****************************************************************************
//*
//*		Chain
//*
//****************************************************************************
//	Erzeugt eine Kette aus aneinander gereihten Texten.
//		("String0"+"\000"+"String1"+"\000"+...+"\000" )
//	iStart	: Ist die Nummer des ersten einzufgenden Textes.
//	iStop	: Ist die Nummer des Textes bis zu Eingefgt wird.
//	pMem	: Hier wird der Zeiger auf die Kette gespeichert.
//			  Dieser Zeiger muss mit delete freigegeben werden.
//	iStart	: Ist die Nummer des ersten einzufgenden Textes.
//	iStop	: Ist die Nummer des Textes bis zu Eingefgt wird.
//	Ergibt Lnge der Kette inklusive NULL-Bytes am Ende.
int CStringChain::Chain(int iStart,int iStop,zchar **pMem)const
{
int		i,j,iLen;
zchar  *pPtr;



	if(iStart<       0)iStart=0;
	if(iStart>m_iCount)iStart=m_iCount;
	if(iStop >m_iCount)iStop =m_iCount;


	if(iStart>=iStop)
		{
		pPtr    = new zchar[2];
		pMem[0] = pPtr;
		pPtr[0] = 0;
		pPtr[1] = 0;
		return 2;
		}


	for(i=iStart,iLen=1;i<iStop;i++)				// Gesamtlnge ermitteln
		{
		iLen += LenNone(i)+1;
		}

	pPtr    = new zchar[iLen];
	pMem[0] = pPtr;

	for(i=iStart;i<iStop;i++)						// Texte kopieren
		{
		j = LenNone(i)+1;
		memcpy(pPtr,GetNone(i),j*sizeof(zchar));
		pPtr+=j;
		}

	pPtr[0]=0;



return iLen;
}

//****************************************************************************
//*
//*		Chain
//*
//****************************************************************************
//	Erzeugt eine Kette aus aneinander gereihten Texten, die mit einem Zeichen
//	getrennt werden.   ("String0"+cSign+"String1"+cSign+...+"\000" )
//	iStart	: Ist die Nummer des ersten einzufgenden Textes.
//	iStop	: Ist die Nummer des Textes bis zu Eingefgt wird.
//	pMem	: Hier wird der Zeiger auf die Kette gespeichert.
//			  Dieser Zeiger muss mit delete freigegeben werden.
//	cSign	: Ist das Zeichen das zwischen die Texte gesetzt wird
//	Ergibt Lnge der Kette inklusive NULL-Bytes am Ende.
int CStringChain::Chain(int iStart,int iStop,zchar **pMem,zchar cSign)const
{
int		i,j,iLen;
zchar  *pPtr;


	if(iStart<       0)iStart=0;
	if(iStart>m_iCount)iStart=m_iCount;
	if(iStop >m_iCount)iStop =m_iCount;


	if(iStart>=iStop)
		{
		pPtr    = new zchar[2];
		pMem[0] = pPtr;
		pPtr[0] = 0;
		pPtr[1] = 0;
		return (cSign)? 1:2;
		}

	for(i=iStart,iLen=1;i<iStop;i++)				// Gesamtlnge ermitteln
		{
		iLen += LenNone(i)+1;
		}

	pPtr    = new zchar[iLen+1];
	pMem[0] = pPtr;

	for(i=iStart;i<iStop;i++)						// Texte kopieren
		{
		j = LenNone(i);
		memcpy(pPtr,GetNone(i),j*sizeof(zchar));
		pPtr[j]=cSign;
		pPtr+=j+1;
		}

	if(cSign){pPtr--;iLen--;}
	*pPtr=0;


return iLen;
}

//****************************************************************************
//*
//*		Chain
//*
//****************************************************************************
//	Erzeugt eine Kette aus aneinander gereihten Texten, die mit einem Zeichen
//	getrennt werden.   ("String0"+cSign+"String1"+cSign+...+"\000" )
//	iStart	: Ist die Nummer des ersten einzufgenden Textes.
//	iStop	: Ist die Nummer des Textes bis zu Eingefgt wird.
//	sStr	: Hier wird die Kette gespeichert.
//	cSign	: Is das Zeichen das zwischen die Texte gesetzt wird
//	Ergibt Lnge der Kette inklusive NULL-Bytes am Ende.
#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)

int CStringChain::Chain(int iStart,int iStop,CString &sStr,zchar cSign)const
{
int		i,j,iLen;
zchar  *pMem;



	if(iStart<       0)iStart=0;
	if(iStart>m_iCount)iStart=m_iCount;
	if(iStop >m_iCount)iStop =m_iCount;


	for(i=iStart,iLen=1;i<iStop;i++)				// Gesamtlnge ermitteln
		{
		iLen += LenNone(i)+1;
		}

	pMem = sStr.GetBuffer(iLen+1);

	for(i=iStart;i<iStop;i++)						// Texte kopieren
		{
		j = LenNone(i);
		memcpy(pMem,GetNone(i),j*sizeof(zchar));
		pMem[j]  = cSign;
		pMem    += j+1;
		}

	if(cSign){pMem--;iLen--;}

	*pMem=0;

	sStr.ReleaseBuffer(iLen);



return iLen;
}

#endif

//****************************************************************************
//*
//*		Memory
//*
//****************************************************************************
//	Erzeugt aus der Textkette eine Textseite. Die einzelnen Texte werden
//	durch "\r\n" bzw. "\n" getrennt. Am Ende wird "\000" angehngt.
//	iStart	: Ist die Nummer des ersten einzufgenden Textes.
//	iStop	: Ist die Nummer des Textes bis zu Eingefgt wird.
//	pMem	: Hier wird der Zeiger auf die Textseite gespeichert.
//			  Dieser Zeiger muss mit delete freigegeben werden.
//	iMax	: Gibt die maximale Lnge der Textseite in Zeichen an.
//	iMode	: Wenn >=0 wird mit "\r\n" getrennt andernfalls mit "\n"
//			  ABS(iMode) ist die Anzahl an Texten ab der ein Zeilenumbruch
//			  eingefgt wird.
//	Ergibt die Lnge der Seite exklusive NULL-Byte am Ende.
int	CStringChain::Memory(int iStart,int iStop,zchar **pMem,int iMax,int iMode)const
{
int		i,j,iNum,iSize,iPos,iLine,iAdd=2;
zchar  *pBuffer,*pEnd=Z("\r\n");


	if(iStart<       0)iStart=0;
	if(iStart>m_iCount)iStart=m_iCount;
	if(iStop >m_iCount)iStop =m_iCount;


	if(iMax ==-1)iMax=0x10000000;					// Keine maximale Lnge
	if(iMode== 0)iMode=1;
	if(iMode < 0)
		{
		iAdd   =  1;
		iMode  = -iMode;
		pEnd   = Z("\n");
		}
													// Speichergre bestimmen
	for(iNum=iStart,iSize=1,iLine=0;iNum<iStop;iNum++)
		{
		iLine++;
		j = LenNone(iNum);
		if(iLine==iMode){j+=iAdd;iLine=0;}
		if(iSize+ j>iMax)break;
		   iSize+=j;
		}

		pBuffer = new zchar[iSize];
	if(!pBuffer)return 0;

	for(i=iStart,iPos=0,iLine=0;i<iNum;i++)			// Texte kopieren
		{
		iLine++;
		j = LenNone(i);
		memcpy(pBuffer+iPos,GetNone(i),j*sizeof(zchar));
		iPos+=j;
		if(iLine!=iMode)continue;

		#ifdef UNICODE
		*(int  *)(pBuffer+iPos)=*(int  *)pEnd;
		#else
		*(short*)(pBuffer+iPos)=*(short*)pEnd;
		#endif

		iPos  += iAdd;
		iLine  = 0;
		}

	 pBuffer[iPos]=0;
	*pMem=pBuffer;


return iPos;
}

//****************************************************************************
//*
//*		DataLength
//*
//****************************************************************************
//	Verndert Datenbereichlnge der Textkette. Das funktioniert nur wenn die
//	Textkette leer ist.
//	iLen	: Ist die neue Datenbereichlnge. Wenn -1 so wird nur die aktuelle
//			  Lnge zurckgegeben.
//	Ergibt die aktuelle Datenbereichlnge.
int CStringChain::DataLength(int iLen)
{


	if(iLen==-1)return m_iDataSize;
	if(m_iCount)return -1;
	if(iLen<0  )return -1;

	m_iDataSize = iLen;


return m_iDataSize;
}


//****************************************************************************
//*
//*		ContainerInit
//*
//****************************************************************************
//	Erstellt einen Container mit der Gre Tiefe
//	iDepth		: Ist die Anzahl der Eintrge fr den Container
//	iMode		: Wenn != 0 wird zwischen Gro und Kleinbuchstaben nicht unterschieden
//	Ergibt die aktuelle Tiefe des der Containers
int CStringChain::ContainerInit(int iDepth,int iMode)
{
int i,j;


	if(iDepth<0)iDepth=16;

	m_iConainerDepth = iDepth;
	m_iConainerMode  = iMode;


	for(i=0;i<m_iConainerDepth && i<m_iCount;i++)
		{
		   j=Find(GetNone(i),i+1,m_iConainerDepth,m_iConainerMode);
		if(j!=-1){Del(j);i=-1;}
		}


	ContainerCut();


return m_iCount;
}

//****************************************************************************
//*
//*		ContainerCheck
//*
//****************************************************************************
//	Sucht nach einen Text im Container und schiebe ihn wenn ntig an die erste Stelle.
//	pStr	: Ist der Zeiger auf den Text.
//	Ergibt 1 wenn der Text nicht im Container war, sonst 0.
int CStringChain::ContainerCheck(const zchar *pStr)
{
int i;


	   i=Find(pStr,0,m_iConainerDepth,m_iConainerMode);
	if(i!=-1)
		{
		Move(i,0);
		return 0;
		}

return 1;
}

//****************************************************************************
//*
//*		Pop
//*
//****************************************************************************
//	Einen Text vom Stapel holen. Der Speicher fr den Text wird automatisch
//	reserviert und muss spter freigegeben werden.
//	pStr	: Hier wird der Zeiger auf den Textspeicher abgelegt.
//	pData	: Hier wird der Datenbereich des Textes gespeichert.
//			  Wenn NULL wird der Datenbereich nicht kopiert.
//	Ergibt die Lnge des Strings bzw. -1 bei einem Fehler
int	CStringChain::Pop(zchar **pStr,void *pData)
{
int	iSize,iPos;


	   iPos=m_iCount-1;
	if(iPos<0)return -1;			 					// Auer Bereich ?

	if(pData)DataGet(iPos,pData);

	iSize = Get(iPos,pStr);
	Del(iPos);


return iSize;
}

//****************************************************************************
//*
//*		Pop
//*
//****************************************************************************
//	Einen Text vom Stapel holen und in einem Puffer ablegen.
//	iLen	: Ist die Gre des Puffers in Zeichen.
//	pStr	: Ist det Zeiger auf den Puffer.
//	pData	: Hier wird der Datenbereich des Textes gespeichert.
//			  Wenn NULL wird der Datenbereich nicht kopiert.
//	Ergibt die Lnge des Strings bzw. -1 bei einem Fehler
int	CStringChain::Pop(int iLen,zchar *pStr,void *pData)
{
int	iSize,iPos;


	   iPos=m_iCount-1;									// Auer Bereich ?
	if(iPos<0)
		{
		if(iLen)pStr[0]=0;
		return -1;
		}

	if(pData)DataGet(iPos,pData);

	   iSize=Get(iPos,pStr,iLen);
	if(iSize!=-1)

	Del(iPos);


return iSize;
}

//****************************************************************************
//*
//*		FifoOut
//*
//****************************************************************************
//	String aus Fifo Speicher holen der Speicher fr den String wird automatisch
//	reserviert und muss spter freigegeben werden.
//	Ergibt die Lnge des Strings bzw. -1 bei einem Fehler
int	CStringChain::FifoOut(zchar **pStr,void *pData)
{
int	iLen;


	if(m_iCount<=0)return -1;			 				// Auer Bereich ?


	if(pData)DataGet(0,pData);
	iLen=Get(0,pStr);
	Del(0);


return iLen;
}

//****************************************************************************
//*
//*		FifoOut
//*
//****************************************************************************
//	String aus Fifo Speicher holen in fixen Speicher
//	Ergibt die Lnge des Strings bzw. -1 bei einem Fehler
int	CStringChain::FifoOut(int iLen,zchar *pStr,void *pData)
{
int	iNum;


	if(m_iCount<=0)
		{
		if(iLen)pStr[0]=0;
		return -1;
		}

	if(pData)
		{
		DataGet(0,pData);
		}

	   iNum =   Get(0,pStr,iLen);
	if(iNum!=-1)Del(0);


return iNum;
}

//****************************************************************************
//*
//*		Plus
//*
//****************************************************************************
//	String an einen String in der Kette anhngen
//	pStr		: Zeiger auf den String
//	iPos		: Ist die Position des Strings der verlngert werden soll
//	pData		: Neuer Datenblock fr den String. (bei 0 kein Tausch)
//	Ergibt die neue Lnge des Strings bzw. -1 bei einem Fehler
int	CStringChain::Plus(const zchar *pStr,int iPos,const void *pData)
{
int		  iLen2;
int		  iLen1;
char	 *pOld;
char	 *pPtr;



	if((uint)iPos>=(uint)m_iCount)						// Neu hinzufgen
		{
		return Add(pStr,m_iCount,pData);
		}

	pOld  = m_pField[iPos];
	iLen1 = *(int*)(pOld-sizeof(int));

	if(!pStr)											// Ist es ein NULL-String
		{
		return iLen1;
		}

		iLen2 = (int)strzlen(pStr);
	if(!iLen2)
		{
		return iLen1;
		}

	NewBlock1(pPtr,pOld,iLen1+iLen2);
	memcpy(pPtr+m_iDataSize,pOld+m_iDataSize        ,iLen1*sizeof(zchar));
	memcpy(pPtr+m_iDataSize+iLen1*sizeof(zchar),pStr,iLen2*sizeof(zchar));

	m_pField[iPos] = pPtr;
	DelBlock(pOld);


return iPos;
}

//****************************************************************************
//*
//*		Plus
//*
//****************************************************************************
//	String an einen String in der Kette anhngen
//	iLen		: Ist die Lnge des an zu hngenden Strings
//	pStr		: Zeiger auf den String
//	iPos		: Ist die Position des Strings der verlngert werden soll
//	pData		: Neuer Datenblock fr den String. (bei 0 kein Tausch)
//	Ergibt die neue Lnge des Strings bzw. -1 bei einem Fehler
int	CStringChain::Plus(int iLen,const zchar *pStr,int iPos,const void *pData)
{
int		  iLen1;
char	 *pOld;
char	 *pPtr;


	if((uint)iPos>=(uint)m_iCount)						// Neu hinzufgen
		{
		return Add(iLen,pStr,m_iCount,pData);
		}


	pOld  = m_pField[iPos];
	iLen1 = *(int*)(pOld-sizeof(int));

	if(!pStr || !iLen)									// Ist es ein NULL-String
		{
		return iLen1;
		}

	NewBlock1(pPtr,pOld,iLen1+iLen);
	memcpy(pPtr+m_iDataSize,pOld+m_iDataSize        ,iLen1*sizeof(zchar));
	memcpy(pPtr+m_iDataSize+iLen1*sizeof(zchar),pStr,iLen *sizeof(zchar));

	m_pField[iPos] = pPtr;
	DelBlock(pOld);


return iPos;
}


//*****************************************************************************
//*
//*		AddFormatEx
//*
//*****************************************************************************
int	_cdecl CStringChain::AddFormatEx(const void *pData,int iPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;



	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;

 	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);		// Formatstring erzeugen

	iRet = Add(sData.iPos,sData.pBuffer,iPos,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}

int	_cdecl CStringChain::AddFormat(const void *pData,int iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(int iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,-1,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(const void *pData,fval *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(fval *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(const void *pData,fabc *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(fabc *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(const void *pData,fint *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(fint *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(const void *pData,fi64 *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(fi64 *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(const void *pData,fu64 *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int	_cdecl CStringChain::AddFormat(fu64 *iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = AddFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

//*****************************************************************************
//*
//*		AddFormatEx
//*
//*****************************************************************************
//	Einen String mittels Formattext einfgen
//	pData	: Zeiger auf die neuen Daten fr den String.
//	pPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Einfgeposition
int	_cdecl CStringChain::AddFormatEx(const void *pData,fval *iPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;


	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);		// Formatstring erzeugen

	   iRet=FindFast(*(int*)pData,sData.iPos,sData.pBuffer,0,m_iCount,(int)iPos);
	if(iRet<0)iRet=0;

	iRet = Add(sData.iPos,sData.pBuffer,iRet,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}

//*****************************************************************************
//*
//*		AddFormatEx
//*
//*****************************************************************************
//	Einen String mittels Formattext einfgen
//	pData	: Zeiger auf die neuen Daten fr den String.
//	pPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Einfgeposition
int	_cdecl CStringChain::AddFormatEx(const void *pData,fabc *iPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;



	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);			// Formatstring erzeugen

	   iRet=FindFast(sData.iPos,sData.pBuffer,0,m_iCount,(int)iPos);
	if(iRet<0)iRet=0;

	iRet = Add(sData.iPos,sData.pBuffer,iRet,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}

//*****************************************************************************
//*
//*		AddFormatEx
//*
//*****************************************************************************
//	Einen String mittels Formattext einfgen
//	pData	: Zeiger auf die neuen Daten fr den String.
//	pPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Einfgeposition
int	_cdecl CStringChain::AddFormatEx(const void *pData,fint *pPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;



	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);			// Formatstring erzeugen

	   iRet=FindFast(*(int*)pData,0,m_iCount,(int)pPos);
	if(iRet<0)iRet=0;

	iRet = Add(sData.iPos,sData.pBuffer,iRet,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}

//*****************************************************************************
//*
//*		AddFormatEx
//*
//*****************************************************************************
//	Einen String mittels Formattext einfgen
//	pData	: Zeiger auf die neuen Daten fr den String.
//	pPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Einfgeposition
int	_cdecl CStringChain::AddFormatEx(const void *pData,fi64 *pPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;



	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);			// Formatstring erzeugen

	   iRet=FindFastQ(*(i64*)pData,0,m_iCount,(int)pPos);
	if(iRet<0)iRet=0;

	iRet = Add(sData.iPos,sData.pBuffer,iRet,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}

//*****************************************************************************
//*
//*		AddFormatEx
//*
//*****************************************************************************
//	Einen String mittels Formattext einfgen
//	pData	: Zeiger auf die neuen Daten fr den String.
//	pPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Einfgeposition
int	_cdecl CStringChain::AddFormatEx(const void *pData,fu64 *pPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;



	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);			// Formatstring erzeugen

	   iRet=FindFastQ(*(i64*)pData,0,m_iCount,(int)pPos);
	if(iRet<0)iRet=0;

	iRet = Add(sData.iPos,sData.pBuffer,iRet,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}

//****************************************************************************
//*
//*		SetFormatEx
//*
//****************************************************************************
//	Tauscht einen String bei Position iPos gehen einen neuen mittel pFormat aus.
//	pData	: Zeiger auf die neuen Daten fr den String.
//	iPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Position in der Kette wo der Text eingefgt wurde
int	_cdecl CStringChain::SetFormatEx(const void *pData,int iPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iRet;



	if((uint)iPos>=(uint)m_iCount)return -1;					// Position ok ?

	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);			// Formatstring erzeugen

	iRet = Set(sData.iPos,sData.pBuffer,iPos,pData);

	if(sData.iNew)delete sData.pBuffer;



return iRet;
}


//****************************************************************************
//*
//*		SetFormat
//*
//****************************************************************************
//	Tauscht einen String bei Position iPos gehen einen neuen mittel pFormat aus.
//	pData	: Zeiger auf die neuen Daten fr den String.
//	iPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Position in der Kette wo der Text eingefgt wurde
int	_cdecl CStringChain::SetFormat(const void *pData ,int   iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = SetFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

//****************************************************************************
//*
//*		SetFormat
//*
//****************************************************************************
//	Tauscht einen String bei Position iPos gehen einen neuen mittel pFormat aus.
//	iPos	: Einfgeposition fr den String (-1 fr das Ende)
//	pFormat	: Zeiger auf Formattext
//	Ergibt die Position in der Kette wo der Text eingefgt wurde
int	_cdecl CStringChain::SetFormat(int iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = SetFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

//****************************************************************************
//*
//*		PlusFormatEx
//*
//****************************************************************************
//	String an einen String in der Kette anhngen mittels Formattext
//	pData	: Zeiger auf die neuen Daten fr den String.
//			  Wenn dieser Zeiger NULL ist werden die Daten nicht getauscht.
//	iPos	: Position des zuverlngerten Strings in der Kette
//	pFormat	: Zeiger auf Formattext
//	Ergibt die neue Lnge des Strings bzw. -1 bei einem Fehler
int	_cdecl CStringChain::PlusFormatEx(const void *pData,int iPos,const zchar *pFormat,va_list pArgs)
{
zchar		cBuffer[1024];
PrintfData	sData;
int			iLen;



	if((uint)iPos>=(uint)m_iCount)return -1;				// Position ok ?

	sData.pBuffer = cBuffer;
	sData.iSize	  = sizeof(cBuffer);
	sData.iPos    = 0;
	sData.iNew	  = 0;


	pprintfz(&sData,StringChainPrintf,pFormat,pArgs);		// Formatstring erzeugen

	iLen = Plus(sData.iPos,sData.pBuffer,iPos,pData);

	if(sData.iNew)delete sData.pBuffer;



return iLen;
}

//****************************************************************************
//*
//*		PlusFormat
//*
//****************************************************************************
//	String an einen String in der Kette anhngen mittels Formattext
//	pData	: Zeiger auf die neuen Daten fr den String.
//			  Wenn dieser Zeiger NULL ist werden die Daten nicht getauscht.
//	iPos	: Position des zuverlngerten Strings in der Kette
//	pFormat	: Zeiger auf Formattext
//	Ergibt die neue Lnge des Strings bzw. -1 bei einem Fehler
int	_cdecl CStringChain::PlusFormat(const void *pData ,int iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = PlusFormatEx(pData,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

//****************************************************************************
//*
//*		PlusFormat
//*
//****************************************************************************
//	String an einen String in der Kette anhngen mittels Formattext
//	iPos	: Position des zuverlngerten Strings in der Kette
//	pFormat	: Zeiger auf Formattext
//	Ergibt die neue Lnge des Strings bzw. -1 bei einem Fehler
int	_cdecl CStringChain::PlusFormat(int iPos,const zchar *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = PlusFormatEx(0,iPos,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

//*****************************************************************************
//*
//*		operator=
//*
//*****************************************************************************
CStringChain &CStringChain::operator=(const CStringChain &sChain)
{

	Del();

	m_iDataSize		 = sChain.m_iDataSize;
	m_iConainerDepth = sChain.m_iConainerDepth;
	m_iConainerMode	 = sChain.m_iConainerMode;
	m_pUserPtr		 = sChain.m_pUserPtr;

	Add((CStringChain*)&sChain);

return *this;
}

//****************************************************************************
//*
//*		Funktionen CStringChain
//*
//****************************************************************************

#ifdef _DEBUG

int	  	CStringChain::Add(          const zchar  *pStr  ,fval *iPos   ,const void *pData){int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);int iNum=FindFast (*(int*)pData,iLen,pStr   ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int	  	CStringChain::Add( int iLen,const zchar  *pStr  ,fval *iPos   ,const void *pData){                                                                   int iNum=FindFast (*(int*)pData,iLen,pStr   ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int	  	CStringChain::Add(          const zchar  *pStr  ,fabc *iPos   ,const void *pData){int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);int iNum=FindFast (             iLen,pStr   ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int	  	CStringChain::Add( int iLen,const zchar  *pStr  ,fabc *iPos   ,const void *pData){                                                                   int iNum=FindFast (             iLen,pStr   ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int	  	CStringChain::Add(          const zchar  *pStr  ,fint *iPos   ,const void *pData){int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);int iNum=FindFast (*(int*)pData             ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add( int iLen,const zchar  *pStr  ,fint *iPos   ,const void *pData){                                                                   int iNum=FindFast (*(int*)pData             ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add(          const zchar  *pStr  ,fi64 *iPos   ,const void *pData){int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);int iNum=FindFastQ(*(i64*)pData             ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add( int iLen,const zchar  *pStr  ,fi64 *iPos   ,const void *pData){                                                                   int iNum=FindFastQ(*(i64*)pData             ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add(          const zchar  *pStr  ,fu64 *iPos   ,const void *pData){int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);int iNum=FindFastQ(*(i64*)pData             ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add( int iLen,const zchar  *pStr  ,fu64 *iPos   ,const void *pData){                                                                   int iNum=FindFastQ(*(i64*)pData             ,0,m_iCount,(int)iPos);return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add(          const zchar  *pStr,CscCmpN pP     ,const void *pData){                                                                   int iNum=FindFast (                  pStr   ,0,m_iCount,0x0C,pP  );return Add(     pStr,iNum,pData);}
int		CStringChain::Add( int iLen,const zchar  *pStr,CscCmpL pP     ,const void *pData){                                                                   int iNum=FindFast (             iLen,pStr   ,0,m_iCount,0x0C,pP  );return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add(          const zchar  *pStr,CscCmpM pP     ,const void *pData){                                                                   int iNum=FindFast (                  pData  ,0,m_iCount,0x0C,pP  );return Add(     pStr,iNum,pData);}
int		CStringChain::Add( int iLen,const zchar  *pStr,CscCmpM pP     ,const void *pData){                                                                   int iNum=FindFast (                  pData  ,0,m_iCount,0x0C,pP  );return Add(iLen,pStr,iNum,pData);}
int		CStringChain::Add(void *pV ,const zchar  *pStr,CscCmpP pP     ,const void *pData){                                                                   int iNum=FindFast (             pV  ,pData  ,0,m_iCount,0x0C,pP  );return Add(     pStr,iNum,pData);}
int		CStringChain::Add(void *pV,int iLen,const zchar *pStr,CscCmpP pP,const void *pData){                                                                 int iNum=FindFast (             pV  ,pData  ,0,m_iCount,0x0C,pP  );return Add(iLen,pStr,iNum,pData);}

int		CStringChain::Add(                    const zchar **pChain,int iPos,const void *pData){return KAdd(          *pChain,iPos,pData);}
int		CStringChain::Add(zchar uKey,         const zchar **pChain,int iPos,const void *pData){return KAdd(uKey,     *pChain,iPos,pData);}
int		CStringChain::Add(zchar uKey,int iMax,const zchar **pChain,int iPos,const void *pData){return KAdd(uKey,iMax,*pChain,iPos,pData);}
int		CStringChain::Add(                          zchar **pChain,int iPos,const void *pData){return KAdd(          *pChain,iPos,pData);}
int		CStringChain::Add(zchar uKey,               zchar **pChain,int iPos,const void *pData){return KAdd(uKey,     *pChain,iPos,pData);}
int		CStringChain::Add(zchar uKey,int iMax,      zchar **pChain,int iPos,const void *pData){return KAdd(uKey,iMax,*pChain,iPos,pData);}

int		CStringChain::DataGet   (int iPos                )const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(int  )                     )return 0;return *(int  *)(m_pField[iPos]);}
i64 	CStringChain::DataGetQ  (int iPos                )const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(i64  )                     )return 0;return *(i64  *)(m_pField[iPos]);}
u16 	CStringChain::DataGetW  (int iPos                )const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(u16  )                     )return 0;return *(u16  *)(m_pField[iPos]);}
u8  	CStringChain::DataGetB  (int iPos                )const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(u8   )                     )return 0;return *(u8   *)(m_pField[iPos]);}
void   *CStringChain::DataGetPtr(int iPos                )const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(void*)                     )return 0;return *(void**)(m_pField[iPos]);}
int 	CStringChain::DataGet   (int iPos,unsigned uIndex)const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(int  )+sizeof(int  )*uIndex)return 0;return *(int  *)(m_pField[iPos]+sizeof(int  )*uIndex);}
i64 	CStringChain::DataGetQ  (int iPos,unsigned uIndex)const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(i64  )+sizeof(i64  )*uIndex)return 0;return *(i64  *)(m_pField[iPos]+sizeof(i64  )*uIndex);}
u16 	CStringChain::DataGetW  (int iPos,unsigned uIndex)const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(u16  )+sizeof(u16  )*uIndex)return 0;return *(u16  *)(m_pField[iPos]+sizeof(u16  )*uIndex);}
u8  	CStringChain::DataGetB  (int iPos,unsigned uIndex)const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(u8   )+sizeof(u8   )*uIndex)return 0;return *(u8   *)(m_pField[iPos]+sizeof(u8   )*uIndex);}
void   *CStringChain::DataGetPtr(int iPos,unsigned uIndex)const{if((uint)iPos>=(uint)m_iCount)return 0;if((uint)m_iDataSize<sizeof(void*)+sizeof(void*)*uIndex)return 0;return *(void**)(m_pField[iPos]+sizeof(void*)*uIndex);}
void   *CStringChain::DataNone  (int iPos)const{                                       return          m_pField[iPos];}
void   *CStringChain::DataEx    (int iPos)const{if((uint)iPos>=(uint)m_iCount)return 0;return          m_pField[iPos]-sizeof(int);}
void   *CStringChain::Data      (int iPos)const{if((uint)iPos>=(uint)m_iCount)return 0;return          m_pField[iPos];}
int     CStringChain::LenNone   (int iPos)const{                                       return  *(int*)(m_pField[iPos]-sizeof(int));}
zchar  *CStringChain::operator[](int iPos)const{if((uint)iPos>=(uint)m_iCount)return 0;return (zchar*)(m_pField[iPos]+m_iDataSize);}
zchar  *CStringChain::Get       (int iPos)const{if((uint)iPos>=(uint)m_iCount)return 0;return (zchar*)(m_pField[iPos]+m_iDataSize);}
zchar  *CStringChain::GetNone   (int iPos)const{                                       return (zchar*)(m_pField[iPos]+m_iDataSize);}
zchar  *CStringChain::Get       (int iPos,int &iLen)const{if((unsigned)iPos>=(unsigned)m_iCount){         iLen=0;return 0;}       iLen =LenNone(iPos);return GetNone(iPos);}
zchar  *CStringChain::GetEx     (int iPos,int *pLen)const{if((unsigned)iPos>=(unsigned)m_iCount){if(pLen)*pLen=0;return 0;}zchar *pName=GetNone(iPos);if(pLen)*pLen=LenNone(iPos);return pName;}
zchar  *CStringChain::Get       (int iPos,int &iLen,int  &iData)const{if((unsigned)iPos>=(unsigned)m_iCount){*(void**)iData=0;iLen=0;return 0;}iData=(m_iDataSize<sizeof(int))? 0:*(int*)m_pField[iPos];iLen =LenNone(iPos);return GetNone(iPos);}
zchar  *CStringChain::Get       (int iPos,int &iLen,void *pData)const{if((unsigned)iPos>=(unsigned)m_iCount){*(void**)pData=0;iLen=0;return 0;}*(void**)pData=m_pField[iPos];iLen =LenNone(iPos);return GetNone(iPos);}
zchar  *CStringChain::Info      (void *pEntry          )const{if(!pEntry){       return 0;}                        return (zchar*)pEntry+m_iDataSize;}
zchar  *CStringChain::Info      (void *pEntry,int &iLen)const{if(!pEntry){iLen=0;return 0;}iLen=((int*)pEntry)[-1];return (zchar*)pEntry+m_iDataSize;}

void	CStringChain::SetUserData(int   iData ){m_pUserPtr=(void*)iData;}
void	CStringChain::SetUserPtr (void *pPtr  ){m_pUserPtr=pPtr ;}
void	CStringChain::GetUserPtr (void *pPtr  )const{*(void**)pPtr=m_pUserPtr;}
int		CStringChain::DataLength ()const{return      m_iDataSize;}
int     CStringChain::GetUserData()const{return (int)m_pUserPtr ;}
void   *CStringChain::GetUserPtr ()const{return      m_pUserPtr ;}
int	    CStringChain::Count      ()const{return      m_iCount   ;}
bool	CStringChain::Check      (int iPos)const{return (unsigned)iPos<(unsigned)m_iCount;}
void   *CStringChain::Alloc      (         const zchar *pStr,int iSize,const void *pData){return Alloc(strzlen(pStr),pStr,iSize,pData);};
void   *CStringChain::Alloc      (int iLen,const zchar *pStr          ,const void *pData){return Alloc(iLen         ,pStr,m_iDataSize,pData);};
void   *CStringChain::Alloc      (	       const zchar *pStr          ,const void *pData){return Alloc(strzlen(pStr),pStr,m_iDataSize,pData);};
void   *CStringChain::Alloc      (int iLen,const zchar *pStr                            ){return Alloc(iLen         ,pStr,m_iDataSize,0);};
void   *CStringChain::Alloc      (		   const zchar *pStr                            ){return Alloc(strzlen(pStr),pStr,m_iDataSize,0);};

int	    CStringChain::ContainerCut(){Del(m_iConainerDepth,m_iCount);return m_iCount;}
int	    CStringChain::Len(int iPos)const{if((unsigned)iPos>=(unsigned)m_iCount)return 0;return *(int*)(m_pField[iPos]-sizeof(int));}
void	CStringChain::Sort(int (__cdecl*pCmp)(void*,const void*,const void*),int iOffset,int iStart,int iStop,int iDirection){Sort(pCmp,0,iOffset,iStart,iStop,iDirection);}

int	  	CStringChain::CAdd   (                    const zchar  *pStr  ,const void *pData){if(ContainerCheck(pStr)){Add (           pStr  ,0,pData);ContainerCut();}return m_iCount;}
int	  	CStringChain::CAdd   (int   iLen,         const zchar  *pStr  ,const void *pData){if(ContainerCheck(pStr)){Add (iLen,      pStr  ,0,pData);ContainerCut();}return m_iCount;}
int	  	CStringChain::CAdd   (                    const zchar **pChain,const void *pData){                         KAdd(          *pChain,0,pData);ContainerInit(m_iConainerDepth,m_iConainerMode);return m_iCount;}
int	  	CStringChain::CAdd   (zchar uKey,         const zchar **pChain,const void *pData){                         KAdd(uKey,     *pChain,0,pData);ContainerInit(m_iConainerDepth,m_iConainerMode);return m_iCount;}
int	    CStringChain::CAdd   (zchar uKey,int iMax,const zchar **pChain,const void *pData){                         KAdd(uKey,iMax,*pChain,0,pData);ContainerInit(m_iConainerDepth,m_iConainerMode);return m_iCount;}
int	  	CStringChain::CAdd   (                          zchar **pChain,const void *pData){                         KAdd(          *pChain,0,pData);ContainerInit(m_iConainerDepth,m_iConainerMode);return m_iCount;}
int	  	CStringChain::CAdd   (zchar uKey,               zchar **pChain,const void *pData){                         KAdd(uKey,     *pChain,0,pData);ContainerInit(m_iConainerDepth,m_iConainerMode);return m_iCount;}
int	    CStringChain::CAdd   (zchar uKey,int iMax,      zchar **pChain,const void *pData){                         KAdd(uKey,iMax,*pChain,0,pData);ContainerInit(m_iConainerDepth,m_iConainerMode);return m_iCount;}

int	  	CStringChain::KAdd   (           const zchar  *pStr  ,int iPos,const void *pData){return KAdd(0,    pStr  ,iPos,pData);}

int	  	CStringChain::Push   (           const zchar  *pStr,const void *pData){return Add(     pStr,m_iCount,pData);}
int	  	CStringChain::Push   (int   iLen,const zchar  *pStr,const void *pData){return Add(iLen,pStr,m_iCount,pData);}

int	  	CStringChain::FifoIn (           const zchar  *pStr,const void *pData){return Add(     pStr,m_iCount,pData);}
int	  	CStringChain::FifoIn (int   iLen,const zchar  *pStr,const void *pData){return Add(iLen,pStr,m_iCount,pData);}

int	  	CStringChain::Find    (const zchar *pStr,int iStart,int iStop,int iMode)const{int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);return Find    (iLen,pStr,iStart,iStop,iMode);}
int	  	CStringChain::FindFast(const zchar *pStr,int iStart,int iStop,int iMode)const{int iLen;if(!pStr){iLen=0;pStr=Z("");}else iLen=(int)strzlen(pStr);return FindFast(iLen,pStr,iStart,iStop,iMode);}

int		CStringChain::Memory  (zchar **pMem,int iMax,int iMode)const{return Memory(0,m_iCount,pMem,iMax,iMode);}
int		CStringChain::Chain   (zchar **pMem                   )const{return Chain (0,m_iCount,pMem           );}
int		CStringChain::Chain   (zchar **pMem,zchar cSign       )const{return Chain (0,m_iCount,pMem,cSign     );}
void  **CStringChain::GetFieldPointer(int **pCount)const{if(pCount)*pCount=(int*)&m_iCount;return (void**)&m_pField;}
void  **CStringChain::GetListPointer()const{return (void**)m_pField;}

#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
int CStringChain::Get	  (int iPos,CString *pStr)const{if((unsigned)iPos>=(unsigned)m_iCount){*pStr="";return -1;}*pStr=m_pField[iPos]+m_iDataSize;return *(int*)(m_pField[iPos]-sizeof(int));}
int CStringChain::Get	  (int iPos,CString &sStr)const{if((unsigned)iPos>=(unsigned)m_iCount){ sStr="";return -1;} sStr=m_pField[iPos]+m_iDataSize;return *(int*)(m_pField[iPos]-sizeof(int));}
int	CStringChain::CAdd    (CString *pStr         ,const void *pData){return CAdd  (pStr->GetLength(),(LPCTSTR)(*pStr)         ,pData);};
int	CStringChain::CAdd    (CString &sStr         ,const void *pData){return CAdd  (sStr .GetLength(),(LPCTSTR)( sStr)         ,pData);};
int	CStringChain::Push    (CString *pStr         ,const void *pData){return Add   (pStr->GetLength(),(LPCTSTR)(*pStr),m_iCount,pData);};
int	CStringChain::Push    (CString &sStr         ,const void *pData){return Add   (sStr .GetLength(),(LPCTSTR)( sStr),m_iCount,pData);};
int	CStringChain::FifoIn  (CString *pStr         ,const void *pData){return Add   (pStr->GetLength(),(LPCTSTR)(*pStr),m_iCount,pData);};
int	CStringChain::FifoIn  (CString &sStr         ,const void *pData){return Add   (sStr .GetLength(),(LPCTSTR)( sStr),m_iCount,pData);};
int	CStringChain::FifoOut (CString *pStr         ,      void *pData){if(m_iCount<=0)return -1;int iLen=LenNone(0);*pStr=GetNone(0);Del(0);return iLen;}
int	CStringChain::FifoOut (CString &pStr         ,      void *pData){if(m_iCount<=0)return -1;int iLen=LenNone(0); pStr=GetNone(0);Del(0);return iLen;}
int	CStringChain::Add     (CString *pStr,int iPos,const void *pData){return Add   (pStr->GetLength(),(LPCTSTR)(*pStr),   iPos,pData);};
int	CStringChain::Add     (CString &sStr,int iPos,const void *pData){return Add   (sStr .GetLength(),(LPCTSTR)( sStr),   iPos,pData);};
int	CStringChain::Set     (CString *pStr,int iPos,const void *pData){return Set   (pStr->GetLength(),(LPCTSTR)(*pStr),   iPos,pData);};
int	CStringChain::Set     (CString &sStr,int iPos,const void *pData){return Set   (sStr .GetLength(),(LPCTSTR)( sStr),   iPos,pData);};
int	CStringChain::Plus    (CString *pStr,int iPos,const void *pData){return Plus  (pStr->GetLength(),(LPCTSTR)(*pStr),   iPos,pData);};
int	CStringChain::Plus    (CString &sStr,int iPos,const void *pData){return Plus  (sStr .GetLength(),(LPCTSTR)( sStr),   iPos,pData);};
int	CStringChain::Pop     (CString *pStr         ,      void *pData){int iNum=m_iCount-1;if(iNum<0)return -1;int iLen=LenNone(iNum);*pStr=GetNone(iNum);Del(iNum);return iLen;}
int	CStringChain::Pop     (CString &pStr         ,      void *pData){int iNum=m_iCount-1;if(iNum<0)return -1;int iLen=LenNone(iNum); pStr=GetNone(iNum);Del(iNum);return iLen;}
int	CStringChain::KAdd    (CString *pStr,zchar cKey,int iPos,const void *pData){return KAdd(cKey,*pStr,iPos,pData);}
int	CStringChain::KAdd    (CString &sStr,zchar cKey,int iPos,const void *pData){return KAdd(cKey, sStr,iPos,pData);}
int	CStringChain::Find    (CString *pStr,int iStart,int iStop,int iMode)const{return Find    (pStr->GetLength(),(LPCTSTR)(*pStr),iStart,iStop,iMode);}
int	CStringChain::Find    (CString &sStr,int iStart,int iStop,int iMode)const{return Find    (sStr .GetLength(),(LPCTSTR)( sStr),iStart,iStop,iMode);}
int	CStringChain::FindFast(CString *pStr,int iStart,int iStop,int iMode)const{return FindFast(pStr->GetLength(),(LPCTSTR)(*pStr),iStart,iStop,iMode);}
int	CStringChain::FindFast(CString &sStr,int iStart,int iStop,int iMode)const{return FindFast(sStr .GetLength(),(LPCTSTR)( sStr),iStart,iStop,iMode);}

int CStringChain::Chain   (                     CString *pStr,zchar cSign)const{return Chain (0     ,m_iCount,*pStr,cSign);}
int CStringChain::Chain   (                     CString &sStr,zchar cSign)const{return Chain (0     ,m_iCount, sStr,cSign);}
int CStringChain::Chain   (int iStart,int iStop,CString *pStr,zchar cSign)const{return Chain (iStart,iStop   ,*pStr,cSign);}


#endif

#endif



